# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: model_registry.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import service as _service
from google.protobuf import service_reflection
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from .scalapb import scalapb_pb2 as scalapb_dot_scalapb__pb2
from . import databricks_pb2 as databricks__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='model_registry.proto',
  package='mlflow',
  syntax='proto2',
  serialized_options=_b('\n\024org.mlflow.api.proto\220\001\001\240\001\001\342?\002\020\001'),
  serialized_pb=_b('\n\x14model_registry.proto\x12\x06mlflow\x1a\x15scalapb/scalapb.proto\x1a\x10\x64\x61tabricks.proto\"\x1f\n\x0fRegisteredModel\x12\x0c\n\x04name\x18\x01 \x01(\t\"\xe5\x01\n\x17RegisteredModelDetailed\x12\x31\n\x10registered_model\x18\x01 \x01(\x0b\x32\x17.mlflow.RegisteredModel\x12\x1a\n\x12\x63reation_timestamp\x18\x02 \x01(\x03\x12\x1e\n\x16last_updated_timestamp\x18\x03 \x01(\x03\x12\x0f\n\x07user_id\x18\x04 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x05 \x01(\t\x12\x35\n\x0flatest_versions\x18\x06 \x03(\x0b\x32\x1c.mlflow.ModelVersionDetailed\"R\n\x0cModelVersion\x12\x31\n\x10registered_model\x18\x01 \x01(\x0b\x32\x17.mlflow.RegisteredModel\x12\x0f\n\x07version\x18\x02 \x01(\x03\"\xa0\x02\n\x14ModelVersionDetailed\x12+\n\rmodel_version\x18\x01 \x01(\x0b\x32\x14.mlflow.ModelVersion\x12\x1a\n\x12\x63reation_timestamp\x18\x02 \x01(\x03\x12\x1e\n\x16last_updated_timestamp\x18\x03 \x01(\x03\x12\x0f\n\x07user_id\x18\x04 \x01(\t\x12\x15\n\rcurrent_stage\x18\x05 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x06 \x01(\t\x12\x0e\n\x06source\x18\x07 \x01(\t\x12\x0e\n\x06run_id\x18\x08 \x01(\t\x12*\n\x06status\x18\t \x01(\x0e\x32\x1a.mlflow.ModelVersionStatus\x12\x16\n\x0estatus_message\x18\n \x01(\t\"\x97\x01\n\x15\x43reateRegisteredModel\x12\x12\n\x04name\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x1a=\n\x08Response\x12\x31\n\x10registered_model\x18\x01 \x01(\x0b\x32\x17.mlflow.RegisteredModel:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xdf\x01\n\x15UpdateRegisteredModel\x12\x37\n\x10registered_model\x18\x01 \x01(\x0b\x32\x17.mlflow.RegisteredModelB\x04\xf8\x86\x19\x01\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x03 \x01(\t\x1a=\n\x08Response\x12\x31\n\x10registered_model\x18\x01 \x01(\x0b\x32\x17.mlflow.RegisteredModel:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\x89\x01\n\x15\x44\x65leteRegisteredModel\x12\x37\n\x10registered_model\x18\x01 \x01(\x0b\x32\x17.mlflow.RegisteredModelB\x04\xf8\x86\x19\x01\x1a\n\n\x08Response:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xd1\x01\n\x19GetRegisteredModelDetails\x12\x37\n\x10registered_model\x18\x01 \x01(\x0b\x32\x17.mlflow.RegisteredModelB\x04\xf8\x86\x19\x01\x1aN\n\x08Response\x12\x42\n\x19registered_model_detailed\x18\x01 \x01(\x0b\x32\x1f.mlflow.RegisteredModelDetailed:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\x94\x01\n\x14ListRegisteredModels\x1aO\n\x08Response\x12\x43\n\x1aregistered_models_detailed\x18\x01 \x03(\x0b\x32\x1f.mlflow.RegisteredModelDetailed:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xd4\x01\n\x11GetLatestVersions\x12\x37\n\x10registered_model\x18\x01 \x01(\x0b\x32\x17.mlflow.RegisteredModelB\x04\xf8\x86\x19\x01\x12\x0e\n\x06stages\x18\x02 \x03(\t\x1aI\n\x08Response\x12=\n\x17model_versions_detailed\x18\x01 \x03(\x0b\x32\x1c.mlflow.ModelVersionDetailed:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xb4\x01\n\x12\x43reateModelVersion\x12\x12\n\x04name\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x14\n\x06source\x18\x02 \x01(\tB\x04\xf8\x86\x19\x01\x12\x0e\n\x06run_id\x18\x03 \x01(\t\x1a\x37\n\x08Response\x12+\n\rmodel_version\x18\x01 \x01(\x0b\x32\x14.mlflow.ModelVersion:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xa4\x01\n\x12UpdateModelVersion\x12\x31\n\rmodel_version\x18\x01 \x01(\x0b\x32\x14.mlflow.ModelVersionB\x04\xf8\x86\x19\x01\x12\r\n\x05stage\x18\x02 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x03 \x01(\t\x1a\n\n\x08Response:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\x80\x01\n\x12\x44\x65leteModelVersion\x12\x31\n\rmodel_version\x18\x01 \x01(\x0b\x32\x14.mlflow.ModelVersionB\x04\xf8\x86\x19\x01\x1a\n\n\x08Response:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xc2\x01\n\x16GetModelVersionDetails\x12\x31\n\rmodel_version\x18\x01 \x01(\x0b\x32\x14.mlflow.ModelVersionB\x04\xf8\x86\x19\x01\x1aH\n\x08Response\x12<\n\x16model_version_detailed\x18\x01 \x01(\x0b\x32\x1c.mlflow.ModelVersionDetailed:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xf6\x01\n\x13SearchModelVersions\x12\x0e\n\x06\x66ilter\x18\x01 \x01(\t\x12\x18\n\x0bmax_results\x18\x02 \x01(\x03:\x03\x31\x30\x30\x12\x10\n\x08order_by\x18\x03 \x03(\t\x12\x12\n\npage_token\x18\x04 \x01(\t\x1a\x62\n\x08Response\x12=\n\x17model_versions_detailed\x18\x01 \x03(\x0b\x32\x1c.mlflow.ModelVersionDetailed\x12\x17\n\x0fnext_page_token\x18\x02 \x01(\t:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\x93\x01\n\x15GetModelVersionStages\x12\x31\n\rmodel_version\x18\x01 \x01(\x0b\x32\x14.mlflow.ModelVersionB\x04\xf8\x86\x19\x01\x1a\x1a\n\x08Response\x12\x0e\n\x06stages\x18\x01 \x03(\t:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\x9e\x01\n\x1aGetModelVersionDownloadUri\x12\x31\n\rmodel_version\x18\x01 \x01(\x0b\x32\x14.mlflow.ModelVersionB\x04\xf8\x86\x19\x01\x1a \n\x08Response\x12\x14\n\x0c\x61rtifact_uri\x18\x01 \x01(\t:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]*}\n\x12ModelVersionStatus\x12\x18\n\x14PENDING_REGISTRATION\x10\x01\x12\x17\n\x13\x46\x41ILED_REGISTRATION\x10\x02\x12\t\n\x05READY\x10\x03\x12\x14\n\x10PENDING_DELETION\x10\x04\x12\x13\n\x0f\x46\x41ILED_DELETION\x10\x05\x32\x8e\x13\n\x14ModelRegistryService\x12\xb6\x01\n\x15\x63reateRegisteredModel\x12\x1d.mlflow.CreateRegisteredModel\x1a&.mlflow.CreateRegisteredModel.Response\"V\xf2\x86\x19R\n6\n\x04POST\x12(/preview/mlflow/registered-models/create\x1a\x04\x08\x02\x10\x00\x10\x01*\x16\x43reate RegisteredModel\x12\xb7\x01\n\x15updateRegisteredModel\x12\x1d.mlflow.UpdateRegisteredModel\x1a&.mlflow.UpdateRegisteredModel.Response\"W\xf2\x86\x19S\n7\n\x05PATCH\x12(/preview/mlflow/registered-models/update\x1a\x04\x08\x02\x10\x00\x10\x01*\x16Update RegisteredModel\x12\xb8\x01\n\x15\x64\x65leteRegisteredModel\x12\x1d.mlflow.DeleteRegisteredModel\x1a&.mlflow.DeleteRegisteredModel.Response\"X\xf2\x86\x19T\n8\n\x06\x44\x45LETE\x12(/preview/mlflow/registered-models/delete\x1a\x04\x08\x02\x10\x00\x10\x01*\x16\x44\x65lete RegisteredModel\x12\xcc\x01\n\x19getRegisteredModelDetails\x12!.mlflow.GetRegisteredModelDetails\x1a*.mlflow.GetRegisteredModelDetails.Response\"`\xf2\x86\x19\\\n;\n\x04POST\x12-/preview/mlflow/registered-models/get-details\x1a\x04\x08\x02\x10\x00\x10\x01*\x1bGet RegisteredModel Details\x12\xaf\x01\n\x14listRegisteredModels\x12\x1c.mlflow.ListRegisteredModels\x1a%.mlflow.ListRegisteredModels.Response\"R\xf2\x86\x19N\n3\n\x03GET\x12&/preview/mlflow/registered-models/list\x1a\x04\x08\x02\x10\x00\x10\x01*\x15List RegisteredModels\x12\xb9\x01\n\x11getLatestVersions\x12\x19.mlflow.GetLatestVersions\x1a\".mlflow.GetLatestVersions.Response\"e\xf2\x86\x19\x61\nC\n\x04POST\x12\x35/preview/mlflow/registered-models/get-latest-versions\x1a\x04\x08\x02\x10\x00\x10\x01*\x18Get Latest ModelVersions\x12\xa7\x01\n\x12\x63reateModelVersion\x12\x1a.mlflow.CreateModelVersion\x1a#.mlflow.CreateModelVersion.Response\"P\xf2\x86\x19L\n3\n\x04POST\x12%/preview/mlflow/model-versions/create\x1a\x04\x08\x02\x10\x00\x10\x01*\x13\x43reate ModelVersion\x12\xa8\x01\n\x12updateModelVersion\x12\x1a.mlflow.UpdateModelVersion\x1a#.mlflow.UpdateModelVersion.Response\"Q\xf2\x86\x19M\n4\n\x05PATCH\x12%/preview/mlflow/model-versions/update\x1a\x04\x08\x02\x10\x00\x10\x01*\x13Update ModelVersion\x12\xa9\x01\n\x12\x64\x65leteModelVersion\x12\x1a.mlflow.DeleteModelVersion\x1a#.mlflow.DeleteModelVersion.Response\"R\xf2\x86\x19N\n5\n\x06\x44\x45LETE\x12%/preview/mlflow/model-versions/delete\x1a\x04\x08\x02\x10\x00\x10\x01*\x13\x44\x65lete ModelVersion\x12\xbd\x01\n\x16getModelVersionDetails\x12\x1e.mlflow.GetModelVersionDetails\x1a\'.mlflow.GetModelVersionDetails.Response\"Z\xf2\x86\x19V\n8\n\x04POST\x12*/preview/mlflow/model-versions/get-details\x1a\x04\x08\x02\x10\x00\x10\x01*\x18Get ModelVersion Details\x12\xaa\x01\n\x13searchModelVersions\x12\x1b.mlflow.SearchModelVersions\x1a$.mlflow.SearchModelVersions.Response\"P\xf2\x86\x19L\n2\n\x03GET\x12%/preview/mlflow/model-versions/search\x1a\x04\x08\x02\x10\x00\x10\x01*\x14Search ModelVersions\x12\xb8\x01\n\x15getModelVersionStages\x12\x1d.mlflow.GetModelVersionStages\x1a&.mlflow.GetModelVersionStages.Response\"X\xf2\x86\x19T\n7\n\x04POST\x12)/preview/mlflow/model-versions/get-stages\x1a\x04\x08\x02\x10\x00\x10\x03*\x17Get ModelVersion Stages\x12\xe1\x01\n\x1agetModelVersionDownloadUri\x12\".mlflow.GetModelVersionDownloadUri\x1a+.mlflow.GetModelVersionDownloadUri.Response\"r\xf2\x86\x19n\n=\n\x04POST\x12//preview/mlflow/model-versions/get-download-uri\x1a\x04\x08\x02\x10\x00\x10\x01*+Get Download URI For ModelVersion ArtifactsB!\n\x14org.mlflow.api.proto\x90\x01\x01\xa0\x01\x01\xe2?\x02\x10\x01')
  ,
  dependencies=[scalapb_dot_scalapb__pb2.DESCRIPTOR,databricks__pb2.DESCRIPTOR,])

_MODELVERSIONSTATUS = _descriptor.EnumDescriptor(
  name='ModelVersionStatus',
  full_name='mlflow.ModelVersionStatus',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='PENDING_REGISTRATION', index=0, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='FAILED_REGISTRATION', index=1, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='READY', index=2, number=3,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PENDING_DELETION', index=3, number=4,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='FAILED_DELETION', index=4, number=5,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=3049,
  serialized_end=3174,
)
_sym_db.RegisterEnumDescriptor(_MODELVERSIONSTATUS)

ModelVersionStatus = enum_type_wrapper.EnumTypeWrapper(_MODELVERSIONSTATUS)
PENDING_REGISTRATION = 1
FAILED_REGISTRATION = 2
READY = 3
PENDING_DELETION = 4
FAILED_DELETION = 5



_REGISTEREDMODEL = _descriptor.Descriptor(
  name='RegisteredModel',
  full_name='mlflow.RegisteredModel',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='mlflow.RegisteredModel.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=73,
  serialized_end=104,
)


_REGISTEREDMODELDETAILED = _descriptor.Descriptor(
  name='RegisteredModelDetailed',
  full_name='mlflow.RegisteredModelDetailed',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='registered_model', full_name='mlflow.RegisteredModelDetailed.registered_model', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='creation_timestamp', full_name='mlflow.RegisteredModelDetailed.creation_timestamp', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='last_updated_timestamp', full_name='mlflow.RegisteredModelDetailed.last_updated_timestamp', index=2,
      number=3, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='user_id', full_name='mlflow.RegisteredModelDetailed.user_id', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='description', full_name='mlflow.RegisteredModelDetailed.description', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='latest_versions', full_name='mlflow.RegisteredModelDetailed.latest_versions', index=5,
      number=6, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=107,
  serialized_end=336,
)


_MODELVERSION = _descriptor.Descriptor(
  name='ModelVersion',
  full_name='mlflow.ModelVersion',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='registered_model', full_name='mlflow.ModelVersion.registered_model', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='version', full_name='mlflow.ModelVersion.version', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=338,
  serialized_end=420,
)


_MODELVERSIONDETAILED = _descriptor.Descriptor(
  name='ModelVersionDetailed',
  full_name='mlflow.ModelVersionDetailed',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='model_version', full_name='mlflow.ModelVersionDetailed.model_version', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='creation_timestamp', full_name='mlflow.ModelVersionDetailed.creation_timestamp', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='last_updated_timestamp', full_name='mlflow.ModelVersionDetailed.last_updated_timestamp', index=2,
      number=3, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='user_id', full_name='mlflow.ModelVersionDetailed.user_id', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='current_stage', full_name='mlflow.ModelVersionDetailed.current_stage', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='description', full_name='mlflow.ModelVersionDetailed.description', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='source', full_name='mlflow.ModelVersionDetailed.source', index=6,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='run_id', full_name='mlflow.ModelVersionDetailed.run_id', index=7,
      number=8, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='status', full_name='mlflow.ModelVersionDetailed.status', index=8,
      number=9, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=1,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='status_message', full_name='mlflow.ModelVersionDetailed.status_message', index=9,
      number=10, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=423,
  serialized_end=711,
)


_CREATEREGISTEREDMODEL_RESPONSE = _descriptor.Descriptor(
  name='Response',
  full_name='mlflow.CreateRegisteredModel.Response',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='registered_model', full_name='mlflow.CreateRegisteredModel.Response.registered_model', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=759,
  serialized_end=820,
)

_CREATEREGISTEREDMODEL = _descriptor.Descriptor(
  name='CreateRegisteredModel',
  full_name='mlflow.CreateRegisteredModel',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='mlflow.CreateRegisteredModel.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=_b('\370\206\031\001'), file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_CREATEREGISTEREDMODEL_RESPONSE, ],
  enum_types=[
  ],
  serialized_options=_b('\342?(\n&com.databricks.rpc.RPC[$this.Response]'),
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=714,
  serialized_end=865,
)


_UPDATEREGISTEREDMODEL_RESPONSE = _descriptor.Descriptor(
  name='Response',
  full_name='mlflow.UpdateRegisteredModel.Response',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='registered_model', full_name='mlflow.UpdateRegisteredModel.Response.registered_model', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=759,
  serialized_end=820,
)

_UPDATEREGISTEREDMODEL = _descriptor.Descriptor(
  name='UpdateRegisteredModel',
  full_name='mlflow.UpdateRegisteredModel',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='registered_model', full_name='mlflow.UpdateRegisteredModel.registered_model', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=_b('\370\206\031\001'), file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='name', full_name='mlflow.UpdateRegisteredModel.name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='description', full_name='mlflow.UpdateRegisteredModel.description', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_UPDATEREGISTEREDMODEL_RESPONSE, ],
  enum_types=[
  ],
  serialized_options=_b('\342?(\n&com.databricks.rpc.RPC[$this.Response]'),
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=868,
  serialized_end=1091,
)


_DELETEREGISTEREDMODEL_RESPONSE = _descriptor.Descriptor(
  name='Response',
  full_name='mlflow.DeleteRegisteredModel.Response',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=759,
  serialized_end=769,
)

_DELETEREGISTEREDMODEL = _descriptor.Descriptor(
  name='DeleteRegisteredModel',
  full_name='mlflow.DeleteRegisteredModel',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='registered_model', full_name='mlflow.DeleteRegisteredModel.registered_model', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=_b('\370\206\031\001'), file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_DELETEREGISTEREDMODEL_RESPONSE, ],
  enum_types=[
  ],
  serialized_options=_b('\342?(\n&com.databricks.rpc.RPC[$this.Response]'),
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1094,
  serialized_end=1231,
)


_GETREGISTEREDMODELDETAILS_RESPONSE = _descriptor.Descriptor(
  name='Response',
  full_name='mlflow.GetRegisteredModelDetails.Response',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='registered_model_detailed', full_name='mlflow.GetRegisteredModelDetails.Response.registered_model_detailed', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1320,
  serialized_end=1398,
)

_GETREGISTEREDMODELDETAILS = _descriptor.Descriptor(
  name='GetRegisteredModelDetails',
  full_name='mlflow.GetRegisteredModelDetails',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='registered_model', full_name='mlflow.GetRegisteredModelDetails.registered_model', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=_b('\370\206\031\001'), file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_GETREGISTEREDMODELDETAILS_RESPONSE, ],
  enum_types=[
  ],
  serialized_options=_b('\342?(\n&com.databricks.rpc.RPC[$this.Response]'),
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1234,
  serialized_end=1443,
)


_LISTREGISTEREDMODELS_RESPONSE = _descriptor.Descriptor(
  name='Response',
  full_name='mlflow.ListRegisteredModels.Response',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='registered_models_detailed', full_name='mlflow.ListRegisteredModels.Response.registered_models_detailed', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1470,
  serialized_end=1549,
)

_LISTREGISTEREDMODELS = _descriptor.Descriptor(
  name='ListRegisteredModels',
  full_name='mlflow.ListRegisteredModels',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[_LISTREGISTEREDMODELS_RESPONSE, ],
  enum_types=[
  ],
  serialized_options=_b('\342?(\n&com.databricks.rpc.RPC[$this.Response]'),
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1446,
  serialized_end=1594,
)


_GETLATESTVERSIONS_RESPONSE = _descriptor.Descriptor(
  name='Response',
  full_name='mlflow.GetLatestVersions.Response',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='model_versions_detailed', full_name='mlflow.GetLatestVersions.Response.model_versions_detailed', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1691,
  serialized_end=1764,
)

_GETLATESTVERSIONS = _descriptor.Descriptor(
  name='GetLatestVersions',
  full_name='mlflow.GetLatestVersions',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='registered_model', full_name='mlflow.GetLatestVersions.registered_model', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=_b('\370\206\031\001'), file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='stages', full_name='mlflow.GetLatestVersions.stages', index=1,
      number=2, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_GETLATESTVERSIONS_RESPONSE, ],
  enum_types=[
  ],
  serialized_options=_b('\342?(\n&com.databricks.rpc.RPC[$this.Response]'),
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1597,
  serialized_end=1809,
)


_CREATEMODELVERSION_RESPONSE = _descriptor.Descriptor(
  name='Response',
  full_name='mlflow.CreateModelVersion.Response',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='model_version', full_name='mlflow.CreateModelVersion.Response.model_version', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1892,
  serialized_end=1947,
)

_CREATEMODELVERSION = _descriptor.Descriptor(
  name='CreateModelVersion',
  full_name='mlflow.CreateModelVersion',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='mlflow.CreateModelVersion.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=_b('\370\206\031\001'), file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='source', full_name='mlflow.CreateModelVersion.source', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=_b('\370\206\031\001'), file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='run_id', full_name='mlflow.CreateModelVersion.run_id', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_CREATEMODELVERSION_RESPONSE, ],
  enum_types=[
  ],
  serialized_options=_b('\342?(\n&com.databricks.rpc.RPC[$this.Response]'),
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1812,
  serialized_end=1992,
)


_UPDATEMODELVERSION_RESPONSE = _descriptor.Descriptor(
  name='Response',
  full_name='mlflow.UpdateModelVersion.Response',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=759,
  serialized_end=769,
)

_UPDATEMODELVERSION = _descriptor.Descriptor(
  name='UpdateModelVersion',
  full_name='mlflow.UpdateModelVersion',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='model_version', full_name='mlflow.UpdateModelVersion.model_version', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=_b('\370\206\031\001'), file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='stage', full_name='mlflow.UpdateModelVersion.stage', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='description', full_name='mlflow.UpdateModelVersion.description', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_UPDATEMODELVERSION_RESPONSE, ],
  enum_types=[
  ],
  serialized_options=_b('\342?(\n&com.databricks.rpc.RPC[$this.Response]'),
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1995,
  serialized_end=2159,
)


_DELETEMODELVERSION_RESPONSE = _descriptor.Descriptor(
  name='Response',
  full_name='mlflow.DeleteModelVersion.Response',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=759,
  serialized_end=769,
)

_DELETEMODELVERSION = _descriptor.Descriptor(
  name='DeleteModelVersion',
  full_name='mlflow.DeleteModelVersion',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='model_version', full_name='mlflow.DeleteModelVersion.model_version', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=_b('\370\206\031\001'), file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_DELETEMODELVERSION_RESPONSE, ],
  enum_types=[
  ],
  serialized_options=_b('\342?(\n&com.databricks.rpc.RPC[$this.Response]'),
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2162,
  serialized_end=2290,
)


_GETMODELVERSIONDETAILS_RESPONSE = _descriptor.Descriptor(
  name='Response',
  full_name='mlflow.GetModelVersionDetails.Response',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='model_version_detailed', full_name='mlflow.GetModelVersionDetails.Response.model_version_detailed', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2370,
  serialized_end=2442,
)

_GETMODELVERSIONDETAILS = _descriptor.Descriptor(
  name='GetModelVersionDetails',
  full_name='mlflow.GetModelVersionDetails',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='model_version', full_name='mlflow.GetModelVersionDetails.model_version', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=_b('\370\206\031\001'), file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_GETMODELVERSIONDETAILS_RESPONSE, ],
  enum_types=[
  ],
  serialized_options=_b('\342?(\n&com.databricks.rpc.RPC[$this.Response]'),
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2293,
  serialized_end=2487,
)


_SEARCHMODELVERSIONS_RESPONSE = _descriptor.Descriptor(
  name='Response',
  full_name='mlflow.SearchModelVersions.Response',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='model_versions_detailed', full_name='mlflow.SearchModelVersions.Response.model_versions_detailed', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='next_page_token', full_name='mlflow.SearchModelVersions.Response.next_page_token', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2593,
  serialized_end=2691,
)

_SEARCHMODELVERSIONS = _descriptor.Descriptor(
  name='SearchModelVersions',
  full_name='mlflow.SearchModelVersions',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='filter', full_name='mlflow.SearchModelVersions.filter', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='max_results', full_name='mlflow.SearchModelVersions.max_results', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=True, default_value=100,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='order_by', full_name='mlflow.SearchModelVersions.order_by', index=2,
      number=3, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='page_token', full_name='mlflow.SearchModelVersions.page_token', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_SEARCHMODELVERSIONS_RESPONSE, ],
  enum_types=[
  ],
  serialized_options=_b('\342?(\n&com.databricks.rpc.RPC[$this.Response]'),
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2490,
  serialized_end=2736,
)


_GETMODELVERSIONSTAGES_RESPONSE = _descriptor.Descriptor(
  name='Response',
  full_name='mlflow.GetModelVersionStages.Response',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='stages', full_name='mlflow.GetModelVersionStages.Response.stages', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2815,
  serialized_end=2841,
)

_GETMODELVERSIONSTAGES = _descriptor.Descriptor(
  name='GetModelVersionStages',
  full_name='mlflow.GetModelVersionStages',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='model_version', full_name='mlflow.GetModelVersionStages.model_version', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=_b('\370\206\031\001'), file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_GETMODELVERSIONSTAGES_RESPONSE, ],
  enum_types=[
  ],
  serialized_options=_b('\342?(\n&com.databricks.rpc.RPC[$this.Response]'),
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2739,
  serialized_end=2886,
)


_GETMODELVERSIONDOWNLOADURI_RESPONSE = _descriptor.Descriptor(
  name='Response',
  full_name='mlflow.GetModelVersionDownloadUri.Response',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='artifact_uri', full_name='mlflow.GetModelVersionDownloadUri.Response.artifact_uri', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2970,
  serialized_end=3002,
)

_GETMODELVERSIONDOWNLOADURI = _descriptor.Descriptor(
  name='GetModelVersionDownloadUri',
  full_name='mlflow.GetModelVersionDownloadUri',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='model_version', full_name='mlflow.GetModelVersionDownloadUri.model_version', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=_b('\370\206\031\001'), file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_GETMODELVERSIONDOWNLOADURI_RESPONSE, ],
  enum_types=[
  ],
  serialized_options=_b('\342?(\n&com.databricks.rpc.RPC[$this.Response]'),
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2889,
  serialized_end=3047,
)

_REGISTEREDMODELDETAILED.fields_by_name['registered_model'].message_type = _REGISTEREDMODEL
_REGISTEREDMODELDETAILED.fields_by_name['latest_versions'].message_type = _MODELVERSIONDETAILED
_MODELVERSION.fields_by_name['registered_model'].message_type = _REGISTEREDMODEL
_MODELVERSIONDETAILED.fields_by_name['model_version'].message_type = _MODELVERSION
_MODELVERSIONDETAILED.fields_by_name['status'].enum_type = _MODELVERSIONSTATUS
_CREATEREGISTEREDMODEL_RESPONSE.fields_by_name['registered_model'].message_type = _REGISTEREDMODEL
_CREATEREGISTEREDMODEL_RESPONSE.containing_type = _CREATEREGISTEREDMODEL
_UPDATEREGISTEREDMODEL_RESPONSE.fields_by_name['registered_model'].message_type = _REGISTEREDMODEL
_UPDATEREGISTEREDMODEL_RESPONSE.containing_type = _UPDATEREGISTEREDMODEL
_UPDATEREGISTEREDMODEL.fields_by_name['registered_model'].message_type = _REGISTEREDMODEL
_DELETEREGISTEREDMODEL_RESPONSE.containing_type = _DELETEREGISTEREDMODEL
_DELETEREGISTEREDMODEL.fields_by_name['registered_model'].message_type = _REGISTEREDMODEL
_GETREGISTEREDMODELDETAILS_RESPONSE.fields_by_name['registered_model_detailed'].message_type = _REGISTEREDMODELDETAILED
_GETREGISTEREDMODELDETAILS_RESPONSE.containing_type = _GETREGISTEREDMODELDETAILS
_GETREGISTEREDMODELDETAILS.fields_by_name['registered_model'].message_type = _REGISTEREDMODEL
_LISTREGISTEREDMODELS_RESPONSE.fields_by_name['registered_models_detailed'].message_type = _REGISTEREDMODELDETAILED
_LISTREGISTEREDMODELS_RESPONSE.containing_type = _LISTREGISTEREDMODELS
_GETLATESTVERSIONS_RESPONSE.fields_by_name['model_versions_detailed'].message_type = _MODELVERSIONDETAILED
_GETLATESTVERSIONS_RESPONSE.containing_type = _GETLATESTVERSIONS
_GETLATESTVERSIONS.fields_by_name['registered_model'].message_type = _REGISTEREDMODEL
_CREATEMODELVERSION_RESPONSE.fields_by_name['model_version'].message_type = _MODELVERSION
_CREATEMODELVERSION_RESPONSE.containing_type = _CREATEMODELVERSION
_UPDATEMODELVERSION_RESPONSE.containing_type = _UPDATEMODELVERSION
_UPDATEMODELVERSION.fields_by_name['model_version'].message_type = _MODELVERSION
_DELETEMODELVERSION_RESPONSE.containing_type = _DELETEMODELVERSION
_DELETEMODELVERSION.fields_by_name['model_version'].message_type = _MODELVERSION
_GETMODELVERSIONDETAILS_RESPONSE.fields_by_name['model_version_detailed'].message_type = _MODELVERSIONDETAILED
_GETMODELVERSIONDETAILS_RESPONSE.containing_type = _GETMODELVERSIONDETAILS
_GETMODELVERSIONDETAILS.fields_by_name['model_version'].message_type = _MODELVERSION
_SEARCHMODELVERSIONS_RESPONSE.fields_by_name['model_versions_detailed'].message_type = _MODELVERSIONDETAILED
_SEARCHMODELVERSIONS_RESPONSE.containing_type = _SEARCHMODELVERSIONS
_GETMODELVERSIONSTAGES_RESPONSE.containing_type = _GETMODELVERSIONSTAGES
_GETMODELVERSIONSTAGES.fields_by_name['model_version'].message_type = _MODELVERSION
_GETMODELVERSIONDOWNLOADURI_RESPONSE.containing_type = _GETMODELVERSIONDOWNLOADURI
_GETMODELVERSIONDOWNLOADURI.fields_by_name['model_version'].message_type = _MODELVERSION
DESCRIPTOR.message_types_by_name['RegisteredModel'] = _REGISTEREDMODEL
DESCRIPTOR.message_types_by_name['RegisteredModelDetailed'] = _REGISTEREDMODELDETAILED
DESCRIPTOR.message_types_by_name['ModelVersion'] = _MODELVERSION
DESCRIPTOR.message_types_by_name['ModelVersionDetailed'] = _MODELVERSIONDETAILED
DESCRIPTOR.message_types_by_name['CreateRegisteredModel'] = _CREATEREGISTEREDMODEL
DESCRIPTOR.message_types_by_name['UpdateRegisteredModel'] = _UPDATEREGISTEREDMODEL
DESCRIPTOR.message_types_by_name['DeleteRegisteredModel'] = _DELETEREGISTEREDMODEL
DESCRIPTOR.message_types_by_name['GetRegisteredModelDetails'] = _GETREGISTEREDMODELDETAILS
DESCRIPTOR.message_types_by_name['ListRegisteredModels'] = _LISTREGISTEREDMODELS
DESCRIPTOR.message_types_by_name['GetLatestVersions'] = _GETLATESTVERSIONS
DESCRIPTOR.message_types_by_name['CreateModelVersion'] = _CREATEMODELVERSION
DESCRIPTOR.message_types_by_name['UpdateModelVersion'] = _UPDATEMODELVERSION
DESCRIPTOR.message_types_by_name['DeleteModelVersion'] = _DELETEMODELVERSION
DESCRIPTOR.message_types_by_name['GetModelVersionDetails'] = _GETMODELVERSIONDETAILS
DESCRIPTOR.message_types_by_name['SearchModelVersions'] = _SEARCHMODELVERSIONS
DESCRIPTOR.message_types_by_name['GetModelVersionStages'] = _GETMODELVERSIONSTAGES
DESCRIPTOR.message_types_by_name['GetModelVersionDownloadUri'] = _GETMODELVERSIONDOWNLOADURI
DESCRIPTOR.enum_types_by_name['ModelVersionStatus'] = _MODELVERSIONSTATUS
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

RegisteredModel = _reflection.GeneratedProtocolMessageType('RegisteredModel', (_message.Message,), dict(
  DESCRIPTOR = _REGISTEREDMODEL,
  __module__ = 'model_registry_pb2'
  # @@protoc_insertion_point(class_scope:mlflow.RegisteredModel)
  ))
_sym_db.RegisterMessage(RegisteredModel)

RegisteredModelDetailed = _reflection.GeneratedProtocolMessageType('RegisteredModelDetailed', (_message.Message,), dict(
  DESCRIPTOR = _REGISTEREDMODELDETAILED,
  __module__ = 'model_registry_pb2'
  # @@protoc_insertion_point(class_scope:mlflow.RegisteredModelDetailed)
  ))
_sym_db.RegisterMessage(RegisteredModelDetailed)

ModelVersion = _reflection.GeneratedProtocolMessageType('ModelVersion', (_message.Message,), dict(
  DESCRIPTOR = _MODELVERSION,
  __module__ = 'model_registry_pb2'
  # @@protoc_insertion_point(class_scope:mlflow.ModelVersion)
  ))
_sym_db.RegisterMessage(ModelVersion)

ModelVersionDetailed = _reflection.GeneratedProtocolMessageType('ModelVersionDetailed', (_message.Message,), dict(
  DESCRIPTOR = _MODELVERSIONDETAILED,
  __module__ = 'model_registry_pb2'
  # @@protoc_insertion_point(class_scope:mlflow.ModelVersionDetailed)
  ))
_sym_db.RegisterMessage(ModelVersionDetailed)

CreateRegisteredModel = _reflection.GeneratedProtocolMessageType('CreateRegisteredModel', (_message.Message,), dict(

  Response = _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), dict(
    DESCRIPTOR = _CREATEREGISTEREDMODEL_RESPONSE,
    __module__ = 'model_registry_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.CreateRegisteredModel.Response)
    ))
  ,
  DESCRIPTOR = _CREATEREGISTEREDMODEL,
  __module__ = 'model_registry_pb2'
  # @@protoc_insertion_point(class_scope:mlflow.CreateRegisteredModel)
  ))
_sym_db.RegisterMessage(CreateRegisteredModel)
_sym_db.RegisterMessage(CreateRegisteredModel.Response)

UpdateRegisteredModel = _reflection.GeneratedProtocolMessageType('UpdateRegisteredModel', (_message.Message,), dict(

  Response = _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), dict(
    DESCRIPTOR = _UPDATEREGISTEREDMODEL_RESPONSE,
    __module__ = 'model_registry_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.UpdateRegisteredModel.Response)
    ))
  ,
  DESCRIPTOR = _UPDATEREGISTEREDMODEL,
  __module__ = 'model_registry_pb2'
  # @@protoc_insertion_point(class_scope:mlflow.UpdateRegisteredModel)
  ))
_sym_db.RegisterMessage(UpdateRegisteredModel)
_sym_db.RegisterMessage(UpdateRegisteredModel.Response)

DeleteRegisteredModel = _reflection.GeneratedProtocolMessageType('DeleteRegisteredModel', (_message.Message,), dict(

  Response = _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), dict(
    DESCRIPTOR = _DELETEREGISTEREDMODEL_RESPONSE,
    __module__ = 'model_registry_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.DeleteRegisteredModel.Response)
    ))
  ,
  DESCRIPTOR = _DELETEREGISTEREDMODEL,
  __module__ = 'model_registry_pb2'
  # @@protoc_insertion_point(class_scope:mlflow.DeleteRegisteredModel)
  ))
_sym_db.RegisterMessage(DeleteRegisteredModel)
_sym_db.RegisterMessage(DeleteRegisteredModel.Response)

GetRegisteredModelDetails = _reflection.GeneratedProtocolMessageType('GetRegisteredModelDetails', (_message.Message,), dict(

  Response = _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), dict(
    DESCRIPTOR = _GETREGISTEREDMODELDETAILS_RESPONSE,
    __module__ = 'model_registry_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.GetRegisteredModelDetails.Response)
    ))
  ,
  DESCRIPTOR = _GETREGISTEREDMODELDETAILS,
  __module__ = 'model_registry_pb2'
  # @@protoc_insertion_point(class_scope:mlflow.GetRegisteredModelDetails)
  ))
_sym_db.RegisterMessage(GetRegisteredModelDetails)
_sym_db.RegisterMessage(GetRegisteredModelDetails.Response)

ListRegisteredModels = _reflection.GeneratedProtocolMessageType('ListRegisteredModels', (_message.Message,), dict(

  Response = _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), dict(
    DESCRIPTOR = _LISTREGISTEREDMODELS_RESPONSE,
    __module__ = 'model_registry_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.ListRegisteredModels.Response)
    ))
  ,
  DESCRIPTOR = _LISTREGISTEREDMODELS,
  __module__ = 'model_registry_pb2'
  # @@protoc_insertion_point(class_scope:mlflow.ListRegisteredModels)
  ))
_sym_db.RegisterMessage(ListRegisteredModels)
_sym_db.RegisterMessage(ListRegisteredModels.Response)

GetLatestVersions = _reflection.GeneratedProtocolMessageType('GetLatestVersions', (_message.Message,), dict(

  Response = _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), dict(
    DESCRIPTOR = _GETLATESTVERSIONS_RESPONSE,
    __module__ = 'model_registry_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.GetLatestVersions.Response)
    ))
  ,
  DESCRIPTOR = _GETLATESTVERSIONS,
  __module__ = 'model_registry_pb2'
  # @@protoc_insertion_point(class_scope:mlflow.GetLatestVersions)
  ))
_sym_db.RegisterMessage(GetLatestVersions)
_sym_db.RegisterMessage(GetLatestVersions.Response)

CreateModelVersion = _reflection.GeneratedProtocolMessageType('CreateModelVersion', (_message.Message,), dict(

  Response = _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), dict(
    DESCRIPTOR = _CREATEMODELVERSION_RESPONSE,
    __module__ = 'model_registry_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.CreateModelVersion.Response)
    ))
  ,
  DESCRIPTOR = _CREATEMODELVERSION,
  __module__ = 'model_registry_pb2'
  # @@protoc_insertion_point(class_scope:mlflow.CreateModelVersion)
  ))
_sym_db.RegisterMessage(CreateModelVersion)
_sym_db.RegisterMessage(CreateModelVersion.Response)

UpdateModelVersion = _reflection.GeneratedProtocolMessageType('UpdateModelVersion', (_message.Message,), dict(

  Response = _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), dict(
    DESCRIPTOR = _UPDATEMODELVERSION_RESPONSE,
    __module__ = 'model_registry_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.UpdateModelVersion.Response)
    ))
  ,
  DESCRIPTOR = _UPDATEMODELVERSION,
  __module__ = 'model_registry_pb2'
  # @@protoc_insertion_point(class_scope:mlflow.UpdateModelVersion)
  ))
_sym_db.RegisterMessage(UpdateModelVersion)
_sym_db.RegisterMessage(UpdateModelVersion.Response)

DeleteModelVersion = _reflection.GeneratedProtocolMessageType('DeleteModelVersion', (_message.Message,), dict(

  Response = _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), dict(
    DESCRIPTOR = _DELETEMODELVERSION_RESPONSE,
    __module__ = 'model_registry_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.DeleteModelVersion.Response)
    ))
  ,
  DESCRIPTOR = _DELETEMODELVERSION,
  __module__ = 'model_registry_pb2'
  # @@protoc_insertion_point(class_scope:mlflow.DeleteModelVersion)
  ))
_sym_db.RegisterMessage(DeleteModelVersion)
_sym_db.RegisterMessage(DeleteModelVersion.Response)

GetModelVersionDetails = _reflection.GeneratedProtocolMessageType('GetModelVersionDetails', (_message.Message,), dict(

  Response = _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), dict(
    DESCRIPTOR = _GETMODELVERSIONDETAILS_RESPONSE,
    __module__ = 'model_registry_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.GetModelVersionDetails.Response)
    ))
  ,
  DESCRIPTOR = _GETMODELVERSIONDETAILS,
  __module__ = 'model_registry_pb2'
  # @@protoc_insertion_point(class_scope:mlflow.GetModelVersionDetails)
  ))
_sym_db.RegisterMessage(GetModelVersionDetails)
_sym_db.RegisterMessage(GetModelVersionDetails.Response)

SearchModelVersions = _reflection.GeneratedProtocolMessageType('SearchModelVersions', (_message.Message,), dict(

  Response = _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), dict(
    DESCRIPTOR = _SEARCHMODELVERSIONS_RESPONSE,
    __module__ = 'model_registry_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.SearchModelVersions.Response)
    ))
  ,
  DESCRIPTOR = _SEARCHMODELVERSIONS,
  __module__ = 'model_registry_pb2'
  # @@protoc_insertion_point(class_scope:mlflow.SearchModelVersions)
  ))
_sym_db.RegisterMessage(SearchModelVersions)
_sym_db.RegisterMessage(SearchModelVersions.Response)

GetModelVersionStages = _reflection.GeneratedProtocolMessageType('GetModelVersionStages', (_message.Message,), dict(

  Response = _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), dict(
    DESCRIPTOR = _GETMODELVERSIONSTAGES_RESPONSE,
    __module__ = 'model_registry_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.GetModelVersionStages.Response)
    ))
  ,
  DESCRIPTOR = _GETMODELVERSIONSTAGES,
  __module__ = 'model_registry_pb2'
  # @@protoc_insertion_point(class_scope:mlflow.GetModelVersionStages)
  ))
_sym_db.RegisterMessage(GetModelVersionStages)
_sym_db.RegisterMessage(GetModelVersionStages.Response)

GetModelVersionDownloadUri = _reflection.GeneratedProtocolMessageType('GetModelVersionDownloadUri', (_message.Message,), dict(

  Response = _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), dict(
    DESCRIPTOR = _GETMODELVERSIONDOWNLOADURI_RESPONSE,
    __module__ = 'model_registry_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.GetModelVersionDownloadUri.Response)
    ))
  ,
  DESCRIPTOR = _GETMODELVERSIONDOWNLOADURI,
  __module__ = 'model_registry_pb2'
  # @@protoc_insertion_point(class_scope:mlflow.GetModelVersionDownloadUri)
  ))
_sym_db.RegisterMessage(GetModelVersionDownloadUri)
_sym_db.RegisterMessage(GetModelVersionDownloadUri.Response)


DESCRIPTOR._options = None
_CREATEREGISTEREDMODEL.fields_by_name['name']._options = None
_CREATEREGISTEREDMODEL._options = None
_UPDATEREGISTEREDMODEL.fields_by_name['registered_model']._options = None
_UPDATEREGISTEREDMODEL._options = None
_DELETEREGISTEREDMODEL.fields_by_name['registered_model']._options = None
_DELETEREGISTEREDMODEL._options = None
_GETREGISTEREDMODELDETAILS.fields_by_name['registered_model']._options = None
_GETREGISTEREDMODELDETAILS._options = None
_LISTREGISTEREDMODELS._options = None
_GETLATESTVERSIONS.fields_by_name['registered_model']._options = None
_GETLATESTVERSIONS._options = None
_CREATEMODELVERSION.fields_by_name['name']._options = None
_CREATEMODELVERSION.fields_by_name['source']._options = None
_CREATEMODELVERSION._options = None
_UPDATEMODELVERSION.fields_by_name['model_version']._options = None
_UPDATEMODELVERSION._options = None
_DELETEMODELVERSION.fields_by_name['model_version']._options = None
_DELETEMODELVERSION._options = None
_GETMODELVERSIONDETAILS.fields_by_name['model_version']._options = None
_GETMODELVERSIONDETAILS._options = None
_SEARCHMODELVERSIONS._options = None
_GETMODELVERSIONSTAGES.fields_by_name['model_version']._options = None
_GETMODELVERSIONSTAGES._options = None
_GETMODELVERSIONDOWNLOADURI.fields_by_name['model_version']._options = None
_GETMODELVERSIONDOWNLOADURI._options = None

_MODELREGISTRYSERVICE = _descriptor.ServiceDescriptor(
  name='ModelRegistryService',
  full_name='mlflow.ModelRegistryService',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  serialized_start=3177,
  serialized_end=5623,
  methods=[
  _descriptor.MethodDescriptor(
    name='createRegisteredModel',
    full_name='mlflow.ModelRegistryService.createRegisteredModel',
    index=0,
    containing_service=None,
    input_type=_CREATEREGISTEREDMODEL,
    output_type=_CREATEREGISTEREDMODEL_RESPONSE,
    serialized_options=_b('\362\206\031R\n6\n\004POST\022(/preview/mlflow/registered-models/create\032\004\010\002\020\000\020\001*\026Create RegisteredModel'),
  ),
  _descriptor.MethodDescriptor(
    name='updateRegisteredModel',
    full_name='mlflow.ModelRegistryService.updateRegisteredModel',
    index=1,
    containing_service=None,
    input_type=_UPDATEREGISTEREDMODEL,
    output_type=_UPDATEREGISTEREDMODEL_RESPONSE,
    serialized_options=_b('\362\206\031S\n7\n\005PATCH\022(/preview/mlflow/registered-models/update\032\004\010\002\020\000\020\001*\026Update RegisteredModel'),
  ),
  _descriptor.MethodDescriptor(
    name='deleteRegisteredModel',
    full_name='mlflow.ModelRegistryService.deleteRegisteredModel',
    index=2,
    containing_service=None,
    input_type=_DELETEREGISTEREDMODEL,
    output_type=_DELETEREGISTEREDMODEL_RESPONSE,
    serialized_options=_b('\362\206\031T\n8\n\006DELETE\022(/preview/mlflow/registered-models/delete\032\004\010\002\020\000\020\001*\026Delete RegisteredModel'),
  ),
  _descriptor.MethodDescriptor(
    name='getRegisteredModelDetails',
    full_name='mlflow.ModelRegistryService.getRegisteredModelDetails',
    index=3,
    containing_service=None,
    input_type=_GETREGISTEREDMODELDETAILS,
    output_type=_GETREGISTEREDMODELDETAILS_RESPONSE,
    serialized_options=_b('\362\206\031\\\n;\n\004POST\022-/preview/mlflow/registered-models/get-details\032\004\010\002\020\000\020\001*\033Get RegisteredModel Details'),
  ),
  _descriptor.MethodDescriptor(
    name='listRegisteredModels',
    full_name='mlflow.ModelRegistryService.listRegisteredModels',
    index=4,
    containing_service=None,
    input_type=_LISTREGISTEREDMODELS,
    output_type=_LISTREGISTEREDMODELS_RESPONSE,
    serialized_options=_b('\362\206\031N\n3\n\003GET\022&/preview/mlflow/registered-models/list\032\004\010\002\020\000\020\001*\025List RegisteredModels'),
  ),
  _descriptor.MethodDescriptor(
    name='getLatestVersions',
    full_name='mlflow.ModelRegistryService.getLatestVersions',
    index=5,
    containing_service=None,
    input_type=_GETLATESTVERSIONS,
    output_type=_GETLATESTVERSIONS_RESPONSE,
    serialized_options=_b('\362\206\031a\nC\n\004POST\0225/preview/mlflow/registered-models/get-latest-versions\032\004\010\002\020\000\020\001*\030Get Latest ModelVersions'),
  ),
  _descriptor.MethodDescriptor(
    name='createModelVersion',
    full_name='mlflow.ModelRegistryService.createModelVersion',
    index=6,
    containing_service=None,
    input_type=_CREATEMODELVERSION,
    output_type=_CREATEMODELVERSION_RESPONSE,
    serialized_options=_b('\362\206\031L\n3\n\004POST\022%/preview/mlflow/model-versions/create\032\004\010\002\020\000\020\001*\023Create ModelVersion'),
  ),
  _descriptor.MethodDescriptor(
    name='updateModelVersion',
    full_name='mlflow.ModelRegistryService.updateModelVersion',
    index=7,
    containing_service=None,
    input_type=_UPDATEMODELVERSION,
    output_type=_UPDATEMODELVERSION_RESPONSE,
    serialized_options=_b('\362\206\031M\n4\n\005PATCH\022%/preview/mlflow/model-versions/update\032\004\010\002\020\000\020\001*\023Update ModelVersion'),
  ),
  _descriptor.MethodDescriptor(
    name='deleteModelVersion',
    full_name='mlflow.ModelRegistryService.deleteModelVersion',
    index=8,
    containing_service=None,
    input_type=_DELETEMODELVERSION,
    output_type=_DELETEMODELVERSION_RESPONSE,
    serialized_options=_b('\362\206\031N\n5\n\006DELETE\022%/preview/mlflow/model-versions/delete\032\004\010\002\020\000\020\001*\023Delete ModelVersion'),
  ),
  _descriptor.MethodDescriptor(
    name='getModelVersionDetails',
    full_name='mlflow.ModelRegistryService.getModelVersionDetails',
    index=9,
    containing_service=None,
    input_type=_GETMODELVERSIONDETAILS,
    output_type=_GETMODELVERSIONDETAILS_RESPONSE,
    serialized_options=_b('\362\206\031V\n8\n\004POST\022*/preview/mlflow/model-versions/get-details\032\004\010\002\020\000\020\001*\030Get ModelVersion Details'),
  ),
  _descriptor.MethodDescriptor(
    name='searchModelVersions',
    full_name='mlflow.ModelRegistryService.searchModelVersions',
    index=10,
    containing_service=None,
    input_type=_SEARCHMODELVERSIONS,
    output_type=_SEARCHMODELVERSIONS_RESPONSE,
    serialized_options=_b('\362\206\031L\n2\n\003GET\022%/preview/mlflow/model-versions/search\032\004\010\002\020\000\020\001*\024Search ModelVersions'),
  ),
  _descriptor.MethodDescriptor(
    name='getModelVersionStages',
    full_name='mlflow.ModelRegistryService.getModelVersionStages',
    index=11,
    containing_service=None,
    input_type=_GETMODELVERSIONSTAGES,
    output_type=_GETMODELVERSIONSTAGES_RESPONSE,
    serialized_options=_b('\362\206\031T\n7\n\004POST\022)/preview/mlflow/model-versions/get-stages\032\004\010\002\020\000\020\003*\027Get ModelVersion Stages'),
  ),
  _descriptor.MethodDescriptor(
    name='getModelVersionDownloadUri',
    full_name='mlflow.ModelRegistryService.getModelVersionDownloadUri',
    index=12,
    containing_service=None,
    input_type=_GETMODELVERSIONDOWNLOADURI,
    output_type=_GETMODELVERSIONDOWNLOADURI_RESPONSE,
    serialized_options=_b('\362\206\031n\n=\n\004POST\022//preview/mlflow/model-versions/get-download-uri\032\004\010\002\020\000\020\001*+Get Download URI For ModelVersion Artifacts'),
  ),
])
_sym_db.RegisterServiceDescriptor(_MODELREGISTRYSERVICE)

DESCRIPTOR.services_by_name['ModelRegistryService'] = _MODELREGISTRYSERVICE

ModelRegistryService = service_reflection.GeneratedServiceType('ModelRegistryService', (_service.Service,), dict(
  DESCRIPTOR = _MODELREGISTRYSERVICE,
  __module__ = 'model_registry_pb2'
  ))

ModelRegistryService_Stub = service_reflection.GeneratedServiceStubType('ModelRegistryService_Stub', (ModelRegistryService,), dict(
  DESCRIPTOR = _MODELREGISTRYSERVICE,
  __module__ = 'model_registry_pb2'
  ))


# @@protoc_insertion_point(module_scope)



import posixpath
from six.moves import urllib

from mlflow.exceptions import MlflowException
from mlflow.protos.databricks_pb2 import INVALID_PARAMETER_VALUE
from mlflow.store.db.db_types import DATABASE_ENGINES
from mlflow.utils.validation import _validate_db_type_string

_INVALID_DB_URI_MSG = "Please refer to https://mlflow.org/docs/latest/tracking.html#storage for " \
                      "format specifications."


def is_local_uri(uri):
    """Returns true if this is a local file path (/foo or file:/foo)."""
    scheme = urllib.parse.urlparse(uri).scheme
    return uri != 'databricks' and (scheme == '' or scheme == 'file')


def is_http_uri(uri):
    scheme = urllib.parse.urlparse(uri).scheme
    return scheme == 'http' or scheme == 'https'


def is_databricks_uri(uri):
    """Databricks URIs look like 'databricks' (default profile) or 'databricks://profile'"""
    scheme = urllib.parse.urlparse(uri).scheme
    return scheme == 'databricks' or uri == 'databricks'


def get_db_profile_from_uri(uri):
    """
    Get the Databricks profile specified by the tracking URI (if any), otherwise
    returns None.
    """
    parsed_uri = urllib.parse.urlparse(uri)
    if parsed_uri.scheme == "databricks":
        return parsed_uri.netloc
    return None


def extract_db_type_from_uri(db_uri):
    """
    Parse the specified DB URI to extract the database type. Confirm the database type is
    supported. If a driver is specified, confirm it passes a plausible regex.
    """
    scheme = urllib.parse.urlparse(db_uri).scheme
    scheme_plus_count = scheme.count('+')

    if scheme_plus_count == 0:
        db_type = scheme
    elif scheme_plus_count == 1:
        db_type, _ = scheme.split('+')
    else:
        error_msg = "Invalid database URI: '%s'. %s" % (db_uri, _INVALID_DB_URI_MSG)
        raise MlflowException(error_msg, INVALID_PARAMETER_VALUE)

    _validate_db_type_string(db_type)

    return db_type


def get_uri_scheme(uri_or_path):
    scheme = urllib.parse.urlparse(uri_or_path).scheme
    if any([scheme.lower().startswith(db) for db in DATABASE_ENGINES]):
        return extract_db_type_from_uri(uri_or_path)
    else:
        return scheme


def append_to_uri_path(uri, *paths):
    """
    Appends the specified POSIX `paths` to the path component of the specified `uri`.

    :param uri: The input URI, represented as a string.
    :param paths: The POSIX paths to append to the specified `uri`'s path component.
    :return: A new URI with a path component consisting of the specified `paths` appended to
             the path component of the specified `uri`.

    >>> uri1 = "s3://root/base/path?param=value"
    >>> uri1 = append_to_uri_path(uri, "some/subpath", "/anotherpath")
    >>> assert uri1 == "s3://root/base/path/some/subpath/anotherpath?param=value"
    >>> uri2 = "a/posix/path"
    >>> uri2 = append_to_uri_path(uri, "/some", "subpath")
    >>> assert uri2 == "a/posixpath/some/subpath"
    """
    path = ""
    for subpath in paths:
        path = _join_posixpaths_and_append_absolute_suffixes(path, subpath)

    parsed_uri = urllib.parse.urlparse(uri)
    if len(parsed_uri.scheme) == 0:
        # If the input URI does not define a scheme, we assume that it is a POSIX path
        # and join it with the specified input paths
        return _join_posixpaths_and_append_absolute_suffixes(uri, path)

    prefix = ""
    if not parsed_uri.path.startswith("/"):
        # For certain URI schemes (e.g., "file:"), urllib's unparse routine does
        # not preserve the relative URI path component properly. In certain cases,
        # urlunparse converts relative paths to absolute paths. We introduce this logic
        # to circumvent urlunparse's erroneous conversion
        prefix = parsed_uri.scheme + ":"
        parsed_uri = parsed_uri._replace(scheme="")

    new_uri_path = _join_posixpaths_and_append_absolute_suffixes(parsed_uri.path, path)
    new_parsed_uri = parsed_uri._replace(path=new_uri_path)
    return prefix + urllib.parse.urlunparse(new_parsed_uri)


def _join_posixpaths_and_append_absolute_suffixes(prefix_path, suffix_path):
    """
    Joins the POSIX path `prefix_path` with the POSIX path `suffix_path`. Unlike posixpath.join(),
    if `suffix_path` is an absolute path, it is appended to prefix_path.

    >>> result1 = _join_posixpaths_and_append_absolute_suffixes("relpath1", "relpath2")
    >>> assert result1 == "relpath1/relpath2"
    >>> result2 = _join_posixpaths_and_append_absolute_suffixes("relpath", "/absolutepath")
    >>> assert result2 == "relpath/absolutepath"
    >>> result3 = _join_posixpaths_and_append_absolute_suffixes("/absolutepath", "relpath")
    >>> assert result3 == "/absolutepath/relpath"
    >>> result4 = _join_posixpaths_and_append_absolute_suffixes("/absolutepath1", "/absolutepath2")
    >>> assert result4 == "/absolutepath1/absolutepath2"
    """
    if len(prefix_path) == 0:
        return suffix_path

    # If the specified prefix path is non-empty, we must relativize the suffix path by removing
    # the leading slash, if present. Otherwise, posixpath.join() would omit the prefix from the
    # joined path
    suffix_path = suffix_path.lstrip(posixpath.sep)
    return posixpath.join(prefix_path, suffix_path)



import yaml

from mlflow.utils import PYTHON_VERSION

_conda_header = """\
name: mlflow-env
channels:
  - defaults
"""


def _mlflow_conda_env(path=None, additional_conda_deps=None, additional_pip_deps=None,
                      additional_conda_channels=None, install_mlflow=True):
    """
    Creates a Conda environment with the specified package channels and dependencies.

    :param path: Local filesystem path where the conda env file is to be written. If unspecified,
                 the conda env will not be written to the filesystem; it will still be returned
                 in dictionary format.
    :param additional_conda_deps: List of additional conda dependencies passed as strings.
    :param additional_pip_deps: List of additional pip dependencies passed as strings.
    :param additional_channels: List of additional conda channels to search when resolving packages.
    :return: ``None`` if ``path`` is specified. Otherwise, the a dictionary representation of the
             Conda environment.
    """
    env = yaml.safe_load(_conda_header)
    env["dependencies"] = ["python={}".format(PYTHON_VERSION)]
    pip_deps = (["mlflow"] if install_mlflow else []) + (
        additional_pip_deps if additional_pip_deps else [])
    if additional_conda_deps is not None:
        env["dependencies"] += additional_conda_deps
    env["dependencies"].append({"pip": pip_deps})
    if additional_conda_channels is not None:
        env["channels"] += additional_conda_channels

    if path is not None:
        with open(path, "w") as out:
            yaml.safe_dump(env, stream=out, default_flow_style=False)
        return None
    else:
        return env



"""
Definitions of click options shared by several CLI commands.
"""
import click

MODEL_PATH = click.option("--model-path", "-m", default=None, metavar="PATH", required=True,
                          help="Path to the model. The path is relative to the run with the given "
                               "run-id or local filesystem path without run-id.")

MODEL_URI = click.option("--model-uri", "-m", default=None, metavar="URI", required=True,
                         help="URI to the model. A local path, a 'runs:/' URI, or a"
                              " remote storage URI (e.g., an 's3://' URI). For more information"
                              " about supported remote URIs for model artifacts, see"
                              " https://mlflow.org/docs/latest/tracking.html"
                              "#artifact-stores")

MLFLOW_HOME = click.option("--mlflow-home", default=None, metavar="PATH",
                           help="Path to local clone of MLflow project. Use for development only.")

RUN_ID = click.option("--run-id", "-r", default=None, required=False, metavar="ID",
                      help="ID of the MLflow run that generated the referenced content.")

NO_CONDA = click.option("--no-conda", is_flag=True,
                        help="If specified, will assume that MLmodel/MLproject is running within "
                             "a Conda environment with the necessary dependencies for "
                             "the current project instead of attempting to create a new "
                             "conda environment.")

INSTALL_MLFLOW = click.option("--install-mlflow", is_flag=True, default=False,
                              help="If specified and there is a conda environment to be activated "
                                   "mlflow will be installed into the environment after it has been"
                                   " activated. The version of installed mlflow will be the same as"
                                   "the one used to invoke this command.")

HOST = click.option("--host", "-h", metavar="HOST", default="127.0.0.1",
                    help="The network address to listen on (default: 127.0.0.1). "
                         "Use 0.0.0.0 to bind to all addresses if you want to access the tracking "
                         "server from other machines.")

PORT = click.option("--port", "-p", default=5000,
                    help="The port to listen on (default: 5000).")

# We use None to disambiguate manually selecting "4"
WORKERS = click.option("--workers", "-w", default=None,
                       help="Number of gunicorn worker processes to handle requests (default: 4).")



from abc import abstractmethod, ABCMeta

from mlflow.entities.model_registry.model_version_stages import ALL_STAGES


class AbstractStore:
    """
    Note:: Experimental: This entity may change or be removed in a future release without warning.
    Abstract class that defines API interfaces for storing Model Registry metadata.
    """

    __metaclass__ = ABCMeta

    def __init__(self):
        """
        Empty constructor. This is deliberately not marked as abstract, else every derived class
        would be forced to create one.
        """
        pass

    # CRUD API for RegisteredModel objects

    @abstractmethod
    def create_registered_model(self, name):
        """
        Create a new registered model in backend store.

        :param name: Name of the new model. This is expected to be unique in the backend store.

        :return: A single object of :py:class:`mlflow.entities.model_registry.RegisteredModel`
        created in the backend.
        """
        pass

    @abstractmethod
    def update_registered_model(self, registered_model, new_name=None, description=None):
        """
        Updates metadata for RegisteredModel entity. Either ``new_name`` or ``description`` should
        be non-None. Backend raises exception if a registered model with given name does not exist.

        :param registered_model: :py:class:`mlflow.entities.model_registry.RegisteredModel` object.

        :param new_name: (Optional) New proposed name for the registered model.
        :param description: (Optional) New description.

        :return: A single updated :py:class:`mlflow.entities.model_registry.RegisteredModel` object.
        """
        pass

    @abstractmethod
    def delete_registered_model(self, registered_model):
        """
        Delete registered model.
        Backend raises exception if a registered model with given name does not exist.

        :param registered_model: :py:class:`mlflow.entities.model_registry.RegisteredModel` object.

        :return: None
        """
        pass

    @abstractmethod
    def list_registered_models(self):
        """
        List of all registered models.

        :return: List of :py:class:`mlflow.entities.model_registry.RegisteredModel` objects.
        """
        pass

    @abstractmethod
    def get_registered_model_details(self, registered_model):
        """
        :param registered_model: :py:class:`mlflow.entities.model_registry.RegisteredModel` object.

        :return: A single :py:class:`mlflow.entities.model_registry.RegisteredModelDetailed` object.
        """
        pass

    @abstractmethod
    def get_latest_versions(self, registered_model, stages=None):
        """
        Latest version models for each requested stage. If no ``stages`` argument is provided,
        returns the latest version for each stage.

        :param registered_model: :py:class:`mlflow.entities.model_registry.RegisteredModel` object.
        :param stages: List of desired stages. If input list is None, return latest versions for
                       for 'Staging' and 'Production' stages.

        :return: List of `:py:class:`mlflow.entities.model_registry.ModelVersionDetailed` objects.
        """
        pass

    # CRUD API for ModelVersion objects

    @abstractmethod
    def create_model_version(self, name, source, run_id):
        """
        Create a new model version from given source and run ID.

        :param name: Name ID for containing registered model.
        :param source: Source path where the MLflow model is stored.
        :param run_id: Run ID from MLflow tracking server that generated the model

        :return: A single object of :py:class:`mlflow.entities.model_registry.ModelVersion`
        created in the backend.
        """
        pass

    @abstractmethod
    def update_model_version(self, model_version, stage=None, description=None):
        """
        Update metadata associated with a model version in backend.

        :param model_version: :py:class:`mlflow.entities.model_registry.ModelVersion` object.
        :param stage: New desired stage for this model version.
        :param description: New description.

        :return: None.
        """
        pass

    @abstractmethod
    def delete_model_version(self, model_version):
        """
        Delete model version in backend.

        :param model_version: :py:class:`mlflow.entities.model_registry.ModelVersion` object.

        :return: None
        """
        pass

    @abstractmethod
    def get_model_version_details(self, model_version):
        """
        :param model_version: :py:class:`mlflow.entities.model_registry.ModelVersion` object.

        :return: A single :py:class:`mlflow.entities.model_registry.ModelVersionDetailed` object.
        """
        pass

    @abstractmethod
    def get_model_version_download_uri(self, model_version):
        """
        Get the download location in Model Registry for this model version.

        :param model_version: :py:class:`mlflow.entities.model_registry.ModelVersion` object.

        :return: A single URI location that allows reads for downloading.
        """
        pass

    @abstractmethod
    def search_model_versions(self, filter_string):
        """
        Search for model versions in backend that satisfy the filter criteria.

        :param filter_string: A filter string expression. Currently supports a single filter
                              condition either name of model like ``name = 'model_name'`` or
                              ``run_id = '...'``.

        :return: PagedList of :py:class:`mlflow.entities.model_registry.ModelVersionDetailed`
                 objects.
        """
        pass

    def get_model_version_stages(self, model_version):  # pylint: disable=unused-argument
        """
        Get all registry stages for the model

        :param model_version: :py:class:`mlflow.entities.model_registry.ModelVersion` object.

        :return: A list of valid stages.
        """
        return ALL_STAGES



"""Add Experiment Tags Table

Revision ID: df50e92ffc5e
Revises: 181f10493468
Create Date: 2019-07-15 17:46:42.704214

"""
from alembic import op
import sqlalchemy as sa
from mlflow.store.tracking.dbmodels.models import SqlExperimentTag

# revision identifiers, used by Alembic.
revision = 'df50e92ffc5e'
down_revision = '181f10493468'
branch_labels = None
depends_on = None


def upgrade():
    op.create_table(SqlExperimentTag.__tablename__,
        sa.Column('key', sa.String(length=250), primary_key=True, nullable=False),
        sa.Column('value', sa.String(length=5000)),
        sa.Column('experiment_id', sa.Integer(), sa.ForeignKey('experiments.experiment_id'), primary_key=True, nullable=False),
        sa.PrimaryKeyConstraint('key', 'experiment_id', name='experiment_tag_pk')
    )


def downgrade():
    pass



from mlflow.entities.model_registry import RegisteredModel
from mlflow.entities.model_registry._model_registry_entity import _ModelRegistryEntity
from mlflow.protos.model_registry_pb2 import ModelVersion as ProtoModelVersion


class ModelVersion(_ModelRegistryEntity):
    """
    .. note::
        Experimental: This entity may change or be removed in a future release without warning.

    MLflow entity for Model Version.
    A model version is uniquely identified using underlying
    :py:class:`mlflow.entities.model_registry.RegisteredModel` and version number.
    """
    def __init__(self, registered_model, version):
        """
        Construct a :py:class:`mlflow.entities.model_registry.RegisteredModel` instance
        :param registered_model: Is an instance of
                                 :py:class:`mlflow.entities.model_registry.RegisteredModel`
        :param version: Integer version
        """
        super(ModelVersion, self).__init__()
        self._registered_model = registered_model
        self._version = version

    @property
    def registered_model(self):
        """An instance of :py:class:`mlflow.entities.model_registry.RegisteredModel`"""
        return self._registered_model

    def get_name(self):
        """String. Unique name within Model Registry."""
        return self.registered_model.name

    @property
    def version(self):
        """Integer version number"""
        return self._version

    # proto mappers
    @classmethod
    def from_proto(cls, proto):
        # input: mlflow.protos.model_registry_pb2.ModelVersion
        # returns: ModelVersion entity
        return cls(RegisteredModel.from_proto(proto.registered_model), proto.version)

    def to_proto(self):
        # returns mlflow.protos.model_registry_pb2.ModelVersion
        model_version = ProtoModelVersion()
        model_version.registered_model.MergeFrom(self.registered_model.to_proto())
        model_version.version = self.version
        return model_version



import os

from mlflow.store.db.db_types import DATABASE_ENGINES
from mlflow.store.model_registry.rest_store import RestStore
from mlflow.tracking._model_registry.registry import ModelRegistryStoreRegistry
from mlflow.tracking._tracking_service.utils import _TRACKING_USERNAME_ENV_VAR, \
    _TRACKING_PASSWORD_ENV_VAR, _TRACKING_TOKEN_ENV_VAR, _TRACKING_INSECURE_TLS_ENV_VAR
from mlflow.utils import rest_utils
from mlflow.utils.databricks_utils import get_databricks_host_creds
from mlflow.utils.uri import get_db_profile_from_uri


# NOTE: in contrast to tracking, we do not support the following ways to specify
# the model registry URI:
#  - via a utility method like tracking.tracking.utils.set_tracking_uri
#  - via environment variables like MLFLOW_TRACKING_URI, MLFLOW_TRACKING_USERNAME, ...
# We do support specifying it
#  - via the ``model_registry_uri`` parameter when creating an ``MlflowClient`` or
#    ``ModelRegistryClient``.
#  - by not specifying anything: in this case we assume the model registry store URI is
#    the same as the tracking store URI. This means Tracking and Model Registry are
#    backed by the same backend DB/Rest server. However, note that we access them via
#    different ``Store`` classes (e.g. ``mlflow.store.tracking.SQLAlchemyStore`` &
#    ``mlflow.store.model_registry.SQLAlchemyStore``).
# This means the following combinations are not supported:
#  - Tracking RestStore & Model Registry RestStore that use different credentials.

# NOTE: SqlAlchemyStore code is commented out here - we can add it back once it's available

def _get_sqlalchemy_store(store_uri):
    from mlflow.store.model_registry.sqlalchemy_store import SqlAlchemyStore
    return SqlAlchemyStore(store_uri)


def _get_rest_store(store_uri, **_):
    def get_default_host_creds():
        return rest_utils.MlflowHostCreds(
            host=store_uri,
            username=os.environ.get(_TRACKING_USERNAME_ENV_VAR),
            password=os.environ.get(_TRACKING_PASSWORD_ENV_VAR),
            token=os.environ.get(_TRACKING_TOKEN_ENV_VAR),
            ignore_tls_verification=os.environ.get(_TRACKING_INSECURE_TLS_ENV_VAR) == 'true',
        )
    return RestStore(get_default_host_creds)


def _get_databricks_rest_store(store_uri, **_):
    profile = get_db_profile_from_uri(store_uri)
    return RestStore(lambda: get_databricks_host_creds(profile))


_model_registry_store_registry = ModelRegistryStoreRegistry()
_model_registry_store_registry.register('databricks', _get_databricks_rest_store)

for scheme in ['http', 'https']:
    _model_registry_store_registry.register(scheme, _get_rest_store)

for scheme in DATABASE_ENGINES:
    _model_registry_store_registry.register(scheme, _get_sqlalchemy_store)

_model_registry_store_registry.register_entrypoints()


def _get_store(store_uri=None):
    return _model_registry_store_registry.get_store(store_uri)



from __future__ import print_function

import os
import json

import click

import mlflow
import mlflow.sagemaker
from mlflow.sagemaker import DEFAULT_IMAGE_NAME as IMAGE
from mlflow.utils import cli_args
import mlflow.models.docker_utils


@click.group("sagemaker")
def commands():
    """
    Serve models on SageMaker.

    To serve a model associated with a run on a tracking server, set the MLFLOW_TRACKING_URI
    environment variable to the URL of the desired server.
    """
    pass


@commands.command("deploy")
@click.option("--app-name", "-a", help="Application name", required=True)
@cli_args.MODEL_URI
@click.option("--execution-role-arn", "-e", default=None, help="SageMaker execution role")
@click.option("--bucket", "-b", default=None, help="S3 bucket to store model artifacts")
@click.option("--image-url", "-i", default=None, help="ECR URL for the Docker image")
@click.option("--region-name", default="us-west-2",
              help="Name of the AWS region in which to deploy the application")
@click.option("--mode", default=mlflow.sagemaker.DEPLOYMENT_MODE_CREATE,
              help="The mode in which to deploy the application."
                   " Must be one of the following: {mds}".format(
                     mds=", ".join(mlflow.sagemaker.DEPLOYMENT_MODES)))
@click.option("--archive", "-ar", is_flag=True,
              help=("If specified, any SageMaker resources that become inactive (i.e as the"
                    " result of an update in {mode_replace} mode) are preserved."
                    " These resources may include unused SageMaker models and endpoint"
                    " configurations that were associated with a prior version of the application"
                    " endpoint. Otherwise, if `--archive` is unspecified, these resources are"
                    " deleted. `--archive` must be specified when deploying asynchronously with"
                    " `--async`.".format(
                      mode_replace=mlflow.sagemaker.DEPLOYMENT_MODE_REPLACE)))
@click.option("--instance-type", "-t", default=mlflow.sagemaker.DEFAULT_SAGEMAKER_INSTANCE_TYPE,
              help="The type of SageMaker ML instance on which to deploy the model. For a list of"
                   " supported instance types, see"
                   " https://aws.amazon.com/sagemaker/pricing/instance-types/.")
@click.option("--instance-count", "-c", default=mlflow.sagemaker.DEFAULT_SAGEMAKER_INSTANCE_COUNT,
              help="The number of SageMaker ML instances on which to deploy the model")
@click.option("--vpc-config", "-v",
              help="Path to a file containing a JSON-formatted VPC configuration. This"
                   " configuration will be used when creating the new SageMaker model associated"
                   " with this application. For more information, see"
                   " https://docs.aws.amazon.com/sagemaker/latest/dg/API_VpcConfig.html")
@click.option("--flavor", "-f", default=None,
              help=("The name of the flavor to use for deployment. Must be one of the following:"
                    " {supported_flavors}. If unspecified, a flavor will be automatically selected"
                    " from the model's available flavors.".format(
                      supported_flavors=mlflow.sagemaker.SUPPORTED_DEPLOYMENT_FLAVORS)))
@click.option("--async", "asynchronous", is_flag=True,
              help=("If specified, this command will return immediately after starting the"
                    " deployment process. It will not wait for the deployment process to complete."
                    " The caller is responsible for monitoring the deployment process via native"
                    " SageMaker APIs or the AWS console."))
@click.option("--timeout", default=1200,
              help=("If the command is executed synchronously, the deployment process will return"
                    " after the specified number of seconds if no definitive result (success or"
                    " failure) is achieved. Once the function returns, the caller is responsible"
                    " for monitoring the health and status of the pending deployment via"
                    " native SageMaker APIs or the AWS console. If the command is executed"
                    " asynchronously using the `--async` flag, this value is ignored."))
def deploy(app_name, model_uri, execution_role_arn, bucket, image_url, region_name, mode, archive,
           instance_type, instance_count, vpc_config, flavor, asynchronous, timeout):
    """
    Deploy model on Sagemaker as a REST API endpoint. Current active AWS account needs to have
    correct permissions setup.

    By default, unless the ``--async`` flag is specified, this command will block until
    either the deployment process completes (definitively succeeds or fails) or the specified
    timeout elapses.

    For more information about the input data formats accepted by the deployed REST API endpoint,
    see the following documentation:
    https://www.mlflow.org/docs/latest/models.html#sagemaker-deployment.
    """
    if vpc_config is not None:
        with open(vpc_config, "r") as f:
            vpc_config = json.load(f)

    mlflow.sagemaker.deploy(app_name=app_name, model_uri=model_uri,
                            execution_role_arn=execution_role_arn, bucket=bucket,
                            image_url=image_url, region_name=region_name, mode=mode,
                            archive=archive, instance_type=instance_type,
                            instance_count=instance_count, vpc_config=vpc_config, flavor=flavor,
                            synchronous=(not asynchronous), timeout_seconds=timeout)


@commands.command("delete")
@click.option("--app-name", "-a", help="Application name", required=True)
@click.option("--region-name", "-r", default="us-west-2",
              help="Name of the AWS region in which to deploy the application.")
@click.option("--archive", "-ar", is_flag=True,
              help=("If specified, resources associated with the application are preserved."
                    " These resources may include unused SageMaker models and endpoint"
                    " configurations that were previously associated with the application endpoint."
                    " Otherwise, if `--archive` is unspecified, these resources are deleted."
                    " `--archive` must be specified when deleting asynchronously with `--async`."))
@click.option("--async", "asynchronous", is_flag=True,
              help=("If specified, this command will return immediately after starting the"
                    " deletion process. It will not wait for the deletion process to complete."
                    " The caller is responsible for monitoring the deletion process via native"
                    " SageMaker APIs or the AWS console."))
@click.option("--timeout", default=1200,
              help=("If the command is executed synchronously, the deployment process will return"
                    " after the specified number of seconds if no definitive result (success or"
                    " failure) is achieved. Once the function returns, the caller is responsible"
                    " for monitoring the health and status of the pending deployment via"
                    " native SageMaker APIs or the AWS console. If the command is executed"
                    " asynchronously using the `--async` flag, this value is ignored."))
def delete(app_name, region_name, archive, asynchronous, timeout):
    """
    Delete the specified application. Unless ``--archive`` is specified, all SageMaker resources
    associated with the application are deleted as well.

    By default, unless the ``--async`` flag is specified, this command will block until
    either the deletion process completes (definitively succeeds or fails) or the specified timeout
    elapses.
    """
    mlflow.sagemaker.delete(
        app_name=app_name, region_name=region_name, archive=archive, synchronous=(not asynchronous),
        timeout_seconds=timeout)


@commands.command("run-local")
@cli_args.MODEL_URI
@click.option("--port", "-p", default=5000, help="Server port. [default: 5000]")
@click.option("--image", "-i", default=IMAGE, help="Docker image name")
@click.option("--flavor", "-f", default=None,
              help=("The name of the flavor to use for local serving. Must be one of the following:"
                    " {supported_flavors}. If unspecified, a flavor will be automatically selected"
                    " from the model's available flavors.".format(
                      supported_flavors=mlflow.sagemaker.SUPPORTED_DEPLOYMENT_FLAVORS)))
def run_local(model_uri, port, image, flavor):
    """
    Serve model locally running in a Sagemaker-compatible Docker container.
    """
    mlflow.sagemaker.run_local(model_uri=model_uri, port=port, image=image, flavor=flavor)


@commands.command("build-and-push-container")
@click.option("--build/--no-build", default=True, help="Build the container if set.")
@click.option("--push/--no-push", default=True, help="Push the container to AWS ECR if set.")
@click.option("--container", "-c", default=IMAGE, help="image name")
@cli_args.MLFLOW_HOME
def build_and_push_container(build, push, container, mlflow_home):
    """
    Build new MLflow Sagemaker image, assign it a name, and push to ECR.

    This function builds an MLflow Docker image.
    The image is built locally and it requires Docker to run.
    The image is pushed to ECR under current active AWS account and to current active AWS region.
    """
    if not (build or push):
        print("skipping both build and push, have nothing to do!")
    if build:
        sagemaker_image_entrypoint = """
        ENTRYPOINT ["python", "-c", "import sys; from mlflow.models import container as C; \
        C._init(sys.argv[1])"]
        """

        def setup_container(_):
            return "\n".join([
                'ENV {disable_env}="false"',
                'RUN python -c "from mlflow.models.container import _install_pyfunc_deps;'
                '_install_pyfunc_deps(None, False)"'
            ])

        mlflow.models.docker_utils._build_image(
            container,
            mlflow_home=os.path.abspath(mlflow_home) if mlflow_home else None,
            entrypoint=sagemaker_image_entrypoint,
            custom_setup_steps_hook=setup_container
        )
    if push:
        mlflow.sagemaker.push_image_to_ecr(container)



from mlflow.entities._mlflow_object import _MLflowObject
from mlflow.protos.service_pb2 import FileInfo as ProtoFileInfo


class FileInfo(_MLflowObject):
    """
    Metadata about a file or directory.
    """

    def __init__(self, path, is_dir, file_size):
        self._path = path
        self._is_dir = is_dir
        self._bytes = file_size

    def __eq__(self, other):
        if type(other) is type(self):
            return self.__dict__ == other.__dict__
        return False

    @property
    def path(self):
        """String path of the file or directory."""
        return self._path

    @property
    def is_dir(self):
        """Whether the FileInfo corresponds to a directory."""
        return self._is_dir

    @property
    def file_size(self):
        """Size of the file or directory. If the FileInfo is a directory, returns None."""
        return self._bytes

    def to_proto(self):
        proto = ProtoFileInfo()
        proto.path = self.path
        proto.is_dir = self.is_dir
        if self.file_size:
            proto.file_size = self.file_size
        return proto

    @classmethod
    def from_proto(cls, proto):
        return cls(proto.path, proto.is_dir, proto.file_size)



from mlflow.tracking.registry import StoreRegistry


class ModelRegistryStoreRegistry(StoreRegistry):
    """Scheme-based registry for model registry store implementations

    This class allows the registration of a function or class to provide an
    implementation for a given scheme of `store_uri` through the `register`
    methods. Implementations declared though the entrypoints
    `mlflow.registry_store` group can be automatically registered through the
    `register_entrypoints` method.

    When instantiating a store through the `get_store` method, the scheme of
    the store URI provided (or inferred from environment) will be used to
    select which implementation to instantiate, which will be called with same
    arguments passed to the `get_store` method.
    """

    def __init__(self):
        super(ModelRegistryStoreRegistry, self).__init__("mlflow.model_registry_store")

    def get_store(self, store_uri=None):
        """Get a store from the registry based on the scheme of store_uri

        :param store_uri: The store URI. If None, it will be inferred from the environment. This URI
                          is used to select which tracking store implementation to instantiate and
                          is passed to the constructor of the implementation.

        :return: An instance of `mlflow.store.model_registry.AbstractStore` that fulfills the
                 store URI requirements.
        """
        from mlflow.tracking._tracking_service import utils
        store_uri = store_uri if store_uri is not None else utils.get_tracking_uri()
        builder = self.get_store_builder(store_uri)
        return builder(store_uri=store_uri)



import os

import posixpath
from six.moves import urllib

from mlflow.entities import FileInfo
from mlflow.store.artifact.artifact_repo import ArtifactRepository


class SFTPArtifactRepository(ArtifactRepository):
    """Stores artifacts as files in a remote directory, via sftp."""

    def __init__(self, artifact_uri, client=None):
        self.uri = artifact_uri
        parsed = urllib.parse.urlparse(artifact_uri)
        self.config = {
            'host': parsed.hostname,
            'port': parsed.port,
            'username': parsed.username,
            'password': parsed.password
        }
        self.path = parsed.path

        if client:
            self.sftp = client
        else:
            import pysftp
            import paramiko

            if self.config['host'] is None:
                self.config['host'] = 'localhost'

            ssh_config = paramiko.SSHConfig()
            user_config_file = os.path.expanduser("~/.ssh/config")
            if os.path.exists(user_config_file):
                with open(user_config_file) as f:
                    ssh_config.parse(f)

            user_config = ssh_config.lookup(self.config['host'])

            if 'hostname' in user_config:
                self.config['host'] = user_config['hostname']

            if self.config.get('username', None) is None and 'user' in user_config:
                self.config['username'] = user_config['user']

            if self.config.get('port', None) is None:
                if 'port' in user_config:
                    self.config['port'] = int(user_config['port'])
                else:
                    self.config['port'] = 22

            if 'identityfile' in user_config:
                self.config['private_key'] = user_config['identityfile'][0]

            self.sftp = pysftp.Connection(**self.config)

        super(SFTPArtifactRepository, self).__init__(artifact_uri)

    def log_artifact(self, local_file, artifact_path=None):
        artifact_dir = posixpath.join(self.path, artifact_path) \
            if artifact_path else self.path
        self.sftp.makedirs(artifact_dir)
        self.sftp.put(local_file,
                      posixpath.join(
                          artifact_dir, os.path.basename(local_file)))

    def log_artifacts(self, local_dir, artifact_path=None):
        artifact_dir = posixpath.join(self.path, artifact_path) \
            if artifact_path else self.path
        self.sftp.makedirs(artifact_dir)
        self.sftp.put_r(local_dir, artifact_dir)

    def _is_directory(self, artifact_path):
        artifact_dir = self.path
        path = posixpath.join(artifact_dir, artifact_path) if artifact_path else artifact_dir
        return self.sftp.isdir(path)

    def list_artifacts(self, path=None):
        artifact_dir = self.path
        list_dir = posixpath.join(artifact_dir, path) if path else artifact_dir
        if not self.sftp.isdir(list_dir):
            return []
        artifact_files = self.sftp.listdir(list_dir)
        infos = []
        for file_name in artifact_files:
            file_path = file_name if path is None else posixpath.join(path, file_name)
            full_file_path = posixpath.join(list_dir, file_name)
            if self.sftp.isdir(full_file_path):
                infos.append(FileInfo(file_path, True, None))
            else:
                infos.append(FileInfo(file_path, False, self.sftp.stat(full_file_path).st_size))
        return infos

    def _download_file(self, remote_file_path, local_path):
        remote_full_path = posixpath.join(self.path, remote_file_path)
        self.sftp.get(remote_full_path, local_path)



"""
The ``mlflow.mleap`` module provides an API for saving Spark MLLib models using the
`MLeap <https://github.com/combust/mleap>`_ persistence mechanism.
A companion module for loading MLflow models with the MLeap flavor format is available in the
``mlflow/java`` package.
"""

from __future__ import absolute_import

import os
import sys
import traceback
from six import reraise

import mlflow
from mlflow.models import Model
from mlflow.exceptions import MlflowException
from mlflow.utils import keyword_only

FLAVOR_NAME = "mleap"


@keyword_only
def log_model(spark_model, sample_input, artifact_path, registered_model_name=None):
    """
    Log a Spark MLLib model in MLeap format as an MLflow artifact
    for the current run. The logged model will have the MLeap flavor.

    NOTE:

        The MLeap model flavor cannot be loaded in Python; it must be loaded using the
        Java module within the ``mlflow/java`` package.

    :param spark_model: Spark PipelineModel to be saved. This model must be MLeap-compatible and
                        cannot contain any custom transformers.
    :param sample_input: Sample PySpark DataFrame input that the model can evaluate. This is
                         required by MLeap for data schema inference.
    :param artifact_path: Run-relative artifact path.
    :param registered_model_name: Note:: Experimental: This argument may change or be removed in a
                                  future release without warning. If given, create a model
                                  version under ``registered_model_name``, also creating a
                                  registered model if one with the given name does not exist.

    >>> import mlflow
    >>> import mlflow.mleap
    >>> import pyspark
    >>> from pyspark.ml import Pipeline
    >>> from pyspark.ml.classification import LogisticRegression
    >>> from pyspark.ml.feature import HashingTF, Tokenizer
    >>># training DataFrame
    >>> training = spark.createDataFrame([
    ...     (0, "a b c d e spark", 1.0),
    ...     (1, "b d", 0.0),
    ...     (2, "spark f g h", 1.0),
    ...     (3, "hadoop mapreduce", 0.0) ], ["id", "text", "label"])
    >>># testing DataFrame
    >>> test_df = spark.createDataFrame([
    ...     (4, "spark i j k"),
    ...     (5, "l m n"),
    ...     (6, "spark hadoop spark"),
    ...     (7, "apache hadoop")], ["id", "text"])
    >>> # Create an MLlib pipeline
    >>> tokenizer = Tokenizer(inputCol="text", outputCol="words")
    >>> hashingTF = HashingTF(inputCol=tokenizer.getOutputCol(), outputCol="features")
    >>> lr = LogisticRegression(maxIter=10, regParam=0.001)
    >>> pipeline = Pipeline(stages=[tokenizer, hashingTF, lr])
    >>> model = pipeline.fit(training)
    >>> #log parameters
    >>> mlflow.log_param("max_iter", 10)
    >>> mlflow.log_param("reg_param", 0.001)
    >>> #log the Spark MLlib model in MLeap format
    >>> mlflow.mleap.log_model(spark_model=model, sample_input=test_df,
    >>>                        artifact_path="mleap-model")
    """
    return Model.log(artifact_path=artifact_path, flavor=mlflow.mleap,
                     spark_model=spark_model, sample_input=sample_input,
                     registered_model_name=registered_model_name)


@keyword_only
def save_model(spark_model, sample_input, path, mlflow_model=Model()):
    """
    Save a Spark MLlib PipelineModel in MLeap format at a local path.
    The saved model will have the MLeap flavor.

    NOTE:

        The MLeap model flavor cannot be loaded in Python; it must be loaded using the
        Java module within the ``mlflow/java`` package.

    :param spark_model: Spark PipelineModel to be saved. This model must be MLeap-compatible and
                  cannot contain any custom transformers.
    :param sample_input: Sample PySpark DataFrame input that the model can evaluate. This is
                         required by MLeap for data schema inference.
    :param path: Local path where the model is to be saved.
    :param mlflow_model: :py:mod:`mlflow.models.Model` to which this flavor is being added.
    """
    add_to_model(mlflow_model=mlflow_model, path=path, spark_model=spark_model,
                 sample_input=sample_input)
    mlflow_model.save(os.path.join(path, "MLmodel"))


@keyword_only
def add_to_model(mlflow_model, path, spark_model, sample_input):
    """
    Add the MLeap flavor to an existing MLflow model.

    :param mlflow_model: :py:mod:`mlflow.models.Model` to which this flavor is being added.
    :param path: Path of the model to which this flavor is being added.
    :param spark_model: Spark PipelineModel to be saved. This model must be MLeap-compatible and
                        cannot contain any custom transformers.
    :param sample_input: Sample PySpark DataFrame input that the model can evaluate. This is
                         required by MLeap for data schema inference.
    """
    from pyspark.ml.pipeline import PipelineModel
    from pyspark.sql import DataFrame
    import mleap.version
    from mleap.pyspark.spark_support import SimpleSparkSerializer  # pylint: disable=unused-variable
    from py4j.protocol import Py4JError

    if not isinstance(spark_model, PipelineModel):
        raise Exception("Not a PipelineModel."
                        " MLeap can save only PipelineModels.")
    if sample_input is None:
        raise Exception("A sample input must be specified in order to add the MLeap flavor.")
    if not isinstance(sample_input, DataFrame):
        raise Exception("The sample input must be a PySpark dataframe of type `{df_type}`".format(
            df_type=DataFrame.__module__))

    # MLeap's model serialization routine requires an absolute output path
    path = os.path.abspath(path)

    mleap_path_full = os.path.join(path, "mleap")
    mleap_datapath_sub = os.path.join("mleap", "model")
    mleap_datapath_full = os.path.join(path, mleap_datapath_sub)
    if os.path.exists(mleap_path_full):
        raise Exception("MLeap model data path already exists at: {path}".format(
            path=mleap_path_full))
    os.makedirs(mleap_path_full)

    dataset = spark_model.transform(sample_input)
    model_path = "file:{mp}".format(mp=mleap_datapath_full)
    try:
        spark_model.serializeToBundle(path=model_path,
                                      dataset=dataset)
    except Py4JError:
        _handle_py4j_error(
                MLeapSerializationException,
                "MLeap encountered an error while serializing the model. Ensure that the model is"
                " compatible with MLeap (i.e does not contain any custom transformers).")

    mlflow_model.add_flavor(FLAVOR_NAME,
                            mleap_version=mleap.version.__version__,
                            model_data=mleap_datapath_sub)


def _handle_py4j_error(reraised_error_type, reraised_error_text):
    """
    Logs information about an exception that is currently being handled
    and reraises it with the specified error text as a message.
    """
    traceback.print_exc()
    tb = sys.exc_info()[2]
    reraise(reraised_error_type, reraised_error_type(reraised_error_text), tb)


class MLeapSerializationException(MlflowException):
    """Exception thrown when a model or DataFrame cannot be serialized in MLeap format"""



from mlflow.entities.view_type import ViewType
from mlflow.exceptions import MlflowException


class LifecycleStage(object):
    ACTIVE = "active"
    DELETED = "deleted"
    _VALID_STAGES = set([ACTIVE, DELETED])

    @classmethod
    def view_type_to_stages(cls, view_type=ViewType.ALL):
        stages = []
        if view_type == ViewType.ACTIVE_ONLY or view_type == ViewType.ALL:
            stages.append(cls.ACTIVE)
        if view_type == ViewType.DELETED_ONLY or view_type == ViewType.ALL:
            stages.append(cls.DELETED)
        return stages

    @classmethod
    def is_valid(cls, lifecycle_stage):
        return lifecycle_stage in cls._VALID_STAGES

    @classmethod
    def matches_view_type(cls, view_type, lifecycle_stage):
        if not cls.is_valid(lifecycle_stage):
            raise MlflowException("Invalid lifecycle stage '%s'" % str(lifecycle_stage))

        if view_type == ViewType.ALL:
            return True
        elif view_type == ViewType.ACTIVE_ONLY:
            return lifecycle_stage == LifecycleStage.ACTIVE
        elif view_type == ViewType.DELETED_ONLY:
            return lifecycle_stage == LifecycleStage.DELETED
        else:
            raise MlflowException("Invalid view type '%s'" % str(view_type))



"""
CLI for azureml module.
"""
from __future__ import print_function

import json

import click

import mlflow.azureml
from mlflow.utils import cli_args, experimental


@click.group("azureml")
def commands():
    """
    Serve models on Azure ML. **These commands require that MLflow be installed with Python 3.**

    To serve a model associated with a run on a tracking server, set the MLFLOW_TRACKING_URI
    environment variable to the URL of the desired server.
    """
    pass


@commands.command("build-image")
@cli_args.MODEL_URI
@click.option("--workspace-name", "-w", required=True,
              help="The name of the Azure Workspace in which to build the image.")
@click.option("--subscription-id", "-s", default=None,
              help=("The subscription id associated with the Azure Workspace in which to build"
                    " the image"))
@click.option("--image-name", "-i", default=None,
              help=("The name to assign the Azure Container Image that is created. If unspecified,"
                    " a unique image name will be generated."))
@click.option("--model-name", "-n", default=None,
              help=("The name to assign the Azure Model that is created. If unspecified,"
                    " a unique image name will be generated."))
@cli_args.MLFLOW_HOME
@click.option("--description", "-d", default=None,
              help=("A string description to associate with the Azure Container Image and the"
                    " Azure Model that are created."))
@click.option("--tags", "-t", default=None,
              help=("A collection of tags, represented as a JSON-formatted dictionary of string"
                    " key-value pairs, to associate with the Azure Container Image and the Azure"
                    " Model that are created. These tags are added to a set of default tags"
                    " that include the model path, the model run id (if specified), and more."))
@experimental
def build_image(model_uri, workspace_name, subscription_id, image_name, model_name,
                mlflow_home, description, tags):
    """
    Register an MLflow model with Azure ML and build an Azure ML ContainerImage for deployment.
    The resulting image can be deployed as a web service to Azure Container Instances (ACI) or
    Azure Kubernetes Service (AKS).

    The resulting Azure ML ContainerImage will contain a webserver that processes model queries.
    For information about the input data formats accepted by this webserver, see the following
    documentation: https://www.mlflow.org/docs/latest/models.html#azureml-deployment.
    """
    # The Azure ML SDK is only compatible with Python 3. However, this CLI should still be
    # accessible for inspection rom Python 2. Therefore, we will only import from the SDK
    # upon command invocation.
    # pylint: disable=import-error
    from azureml.core import Workspace

    workspace = Workspace.get(name=workspace_name, subscription_id=subscription_id)
    if tags is not None:
        tags = json.loads(tags)
    mlflow.azureml.build_image(
            model_uri=model_uri, workspace=workspace, image_name=image_name, model_name=model_name,
            mlflow_home=mlflow_home, description=description, tags=tags, synchronous=True)



# -*- coding: utf-8 -*-

"""Export and import of generic R models.

This module defines generic filesystem format for R models and provides utilities
for saving and loading to and from this format. The format is self contained in the sense
that it includes all necessary information for anyone to load it and use it. Dependencies
are either stored directly with the model or referenced via a Conda environment.

The convention for rfunc models is to have a ``predict`` method or function with the following
signature::

    predict(data: DataFrame) -> DataFrame

This convention is relied on by other MLflow components.

Rfunc model format is defined as a directory structure containing all required data, code, and
configuration:

.. code::

    ./dst-path/
            ./MLmodel: configuration

It must contain MLmodel file in its root with "r_function" format.

Example:

.. code:: shell

  >tree R/mlflow/R/inst/examples/R/lm/model
  ├── MLmodel
  └── r_model.bin

  >cat R/mlflow/R/inst/examples/R/lm/model/MLmodel
  time_created: 1.5337659e+09
  flavors:
    r_function:
      version: 0.1.0
      model: r_model.bin

"""



import os
import shlex
import sys

from flask import Flask, send_from_directory

from mlflow.server import handlers
from mlflow.server.handlers import get_artifact_handler, STATIC_PREFIX_ENV_VAR, _add_static_prefix
from mlflow.utils.process import exec_cmd

# NB: These are intenrnal environment variables used for communication between
# the cli and the forked gunicorn processes.
BACKEND_STORE_URI_ENV_VAR = "_MLFLOW_SERVER_FILE_STORE"
ARTIFACT_ROOT_ENV_VAR = "_MLFLOW_SERVER_ARTIFACT_ROOT"
PROMETHEUS_EXPORTER_ENV_VAR = "prometheus_multiproc_dir"

REL_STATIC_DIR = "js/build"

app = Flask(__name__, static_folder=REL_STATIC_DIR)
STATIC_DIR = os.path.join(app.root_path, REL_STATIC_DIR)


for http_path, handler, methods in handlers.get_endpoints():
    app.add_url_rule(http_path, handler.__name__, handler, methods=methods)

if os.getenv(PROMETHEUS_EXPORTER_ENV_VAR):
    from mlflow.server.prometheus_exporter import activate_prometheus_exporter
    prometheus_metrics_path = os.getenv(PROMETHEUS_EXPORTER_ENV_VAR)
    if not os.path.exists(prometheus_metrics_path):
        os.makedirs(prometheus_metrics_path)
    activate_prometheus_exporter(app)


# Serve the "get-artifact" route.
@app.route(_add_static_prefix('/get-artifact'))
def serve_artifacts():
    return get_artifact_handler()


# We expect the react app to be built assuming it is hosted at /static-files, so that requests for
# CSS/JS resources will be made to e.g. /static-files/main.css and we can handle them here.
@app.route(_add_static_prefix('/static-files/<path:path>'))
def serve_static_file(path):
    return send_from_directory(STATIC_DIR, path)


# Serve the index.html for the React App for all other routes.
@app.route(_add_static_prefix('/'))
def serve():
    return send_from_directory(STATIC_DIR, 'index.html')


def _build_waitress_command(waitress_opts, host, port):
    opts = shlex.split(waitress_opts) if waitress_opts else []
    return ['waitress-serve'] + \
        opts + [
            "--host=%s" % host,
            "--port=%s" % port,
            "--ident=mlflow",
            "mlflow.server:app"
    ]


def _build_gunicorn_command(gunicorn_opts, host, port, workers):
    bind_address = "%s:%s" % (host, port)
    opts = shlex.split(gunicorn_opts) if gunicorn_opts else []
    return ["gunicorn"] + opts + ["-b", bind_address, "-w", "%s" % workers, "mlflow.server:app"]


def _run_server(file_store_path, default_artifact_root, host, port, static_prefix=None,
                workers=None, gunicorn_opts=None, waitress_opts=None, expose_prometheus=None):
    """
    Run the MLflow server, wrapping it in gunicorn or waitress on windows
    :param static_prefix: If set, the index.html asset will be served from the path static_prefix.
                          If left None, the index.html asset will be served from the root path.
    :return: None
    """
    env_map = {}
    if file_store_path:
        env_map[BACKEND_STORE_URI_ENV_VAR] = file_store_path
    if default_artifact_root:
        env_map[ARTIFACT_ROOT_ENV_VAR] = default_artifact_root
    if static_prefix:
        env_map[STATIC_PREFIX_ENV_VAR] = static_prefix

    if expose_prometheus:
        env_map[PROMETHEUS_EXPORTER_ENV_VAR] = expose_prometheus

    # TODO: eventually may want waitress on non-win32
    if sys.platform == 'win32':
        full_command = _build_waitress_command(waitress_opts, host, port)
    else:
        full_command = _build_gunicorn_command(gunicorn_opts, host, port, workers or 4)
    exec_cmd(full_command, env=env_map, stream_output=True)



"""add metric step

Revision ID: 451aebb31d03
Revises:
Create Date: 2019-04-22 15:29:24.921354

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = '451aebb31d03'
down_revision = None
branch_labels = None
depends_on = None


def upgrade():
    op.add_column('metrics', sa.Column('step', sa.BigInteger(), nullable=False, server_default='0'))
    # Use batch mode so that we can run "ALTER TABLE" statements against SQLite
    # databases (see more info at https://alembic.sqlalchemy.org/en/latest/
    # batch.html#running-batch-migrations-for-sqlite-and-other-databases)
    with op.batch_alter_table("metrics") as batch_op:
        batch_op.drop_constraint(constraint_name='metric_pk', type_="primary")
        batch_op.create_primary_key(
            constraint_name='metric_pk',
            columns=['key', 'timestamp', 'step', 'run_uuid', 'value'])


def downgrade():
    # This migration cannot safely be downgraded; once metric data with the same
    # (key, timestamp, run_uuid, value) are inserted (differing only in their `step`), we cannot
    # revert to a schema where (key, timestamp, run_uuid, value) is the metric primary key.
    pass



import os
import posixpath

from mlflow.exceptions import MlflowException
from mlflow.models import Model
from mlflow.protos.databricks_pb2 import RESOURCE_DOES_NOT_EXIST
from mlflow.tracking.artifact_utils import _download_artifact_from_uri


def _get_flavor_configuration(model_path, flavor_name):
    """
    Obtains the configuration for the specified flavor from the specified
    MLflow model path. If the model does not contain the specified flavor,
    an exception will be thrown.

    :param model_path: The path to the root directory of the MLflow model for which to load
                       the specified flavor configuration.
    :param flavor_name: The name of the flavor configuration to load.
    :return: The flavor configuration as a dictionary.
    """
    model_configuration_path = os.path.join(model_path, "MLmodel")
    if not os.path.exists(model_configuration_path):
        raise MlflowException(
            "Could not find an \"MLmodel\" configuration file at \"{model_path}\"".format(
                model_path=model_path),
            RESOURCE_DOES_NOT_EXIST)

    model_conf = Model.load(model_configuration_path)
    if flavor_name not in model_conf.flavors:
        raise MlflowException(
            "Model does not have the \"{flavor_name}\" flavor".format(flavor_name=flavor_name),
            RESOURCE_DOES_NOT_EXIST)
    conf = model_conf.flavors[flavor_name]
    return conf


def _get_flavor_configuration_from_uri(model_uri, flavor_name):
    """
    Obtains the configuration for the specified flavor from the specified
    MLflow model uri. If the model does not contain the specified flavor,
    an exception will be thrown.

    :param model_uri: The path to the root directory of the MLflow model for which to load
                       the specified flavor configuration.
    :param flavor_name: The name of the flavor configuration to load.
    :return: The flavor configuration as a dictionary.
    """
    try:
        ml_model_file = _download_artifact_from_uri(
            artifact_uri=posixpath.join(model_uri, "MLmodel"))
    except Exception as ex:
        raise MlflowException(
            "Failed to download an \"MLmodel\" model file from \"{model_uri}\": {ex}".format(
                model_uri=model_uri, ex=ex),
            RESOURCE_DOES_NOT_EXIST)
    model_conf = Model.load(ml_model_file)
    if flavor_name not in model_conf.flavors:
        raise MlflowException(
            "Model does not have the \"{flavor_name}\" flavor".format(flavor_name=flavor_name),
            RESOURCE_DOES_NOT_EXIST)
    return model_conf.flavors[flavor_name]



"""allow nulls for metric values

Revision ID: 181f10493468
Revises: 90e64c465722
Create Date: 2019-07-10 22:40:18.787993

"""
from alembic import op
import sqlalchemy as sa

# revision identifiers, used by Alembic.
revision = '181f10493468'
down_revision = '90e64c465722'
branch_labels = None
depends_on = None


def upgrade():
    with op.batch_alter_table("metrics") as batch_op:
        batch_op.alter_column("value", type_=sa.types.Float(precision=53), nullable=False)
        batch_op.add_column(sa.Column('is_nan', sa.Boolean(create_constraint=False), nullable=False,
                                      server_default='0'))
        batch_op.drop_constraint(constraint_name='metric_pk', type_="primary")
        batch_op.create_primary_key(
            constraint_name='metric_pk',
            columns=['key', 'timestamp', 'step', 'run_uuid', 'value', 'is_nan'])


def downgrade():
    pass
